@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: application.cpp,v 1.18 2004/06/08 20:39:32 bherger Exp $
+** $Id: application.cpp,v 1.34 2005/03/22 05:42:53 bherger Exp $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -33,6 +33,7 @@ using std::endl;
 #include <qmultilineedit.h>
 #include <qfile.h>
 #include <qfiledialog.h>
+#include <qfontdialog.h>
 #include <qstatusbar.h>
 #include <qmessagebox.h>
 #include <qprinter.h>
@@ -76,6 +77,7 @@ using std::endl;
 #include "myfiledialog.h"
 #include "xruler.h"
 #include "dyk.h"
+#include "charsel.h"
 
 // main window icon
 #include "xdc.xpm"
@@ -143,11 +145,13 @@ using std::endl;
 // end Arrows
 
 // defined in main.cpp
-extern QString RingDir;
+//extern QString RingDir, HomeDir;
 
 ApplicationWindow::ApplicationWindow()
     : QMainWindow( 0, "XDrawChem main window", WDestructiveClose )
 {
+  setFont(preferences.getMainFont());
+
   setIcon(QPixmap(XDC_icon));
   OBGetFilters();
 
@@ -512,10 +516,11 @@ ApplicationWindow::ApplicationWindow()
   myfactory->setPixmap("ringtoolicon", ringIcon );
   QToolButton * drawRing
     = new QToolButton( ringIcon, tr("Draw ring"), QString::null,
-		       this, SLOT(MakeRingDialog()), drawTools, 
+		       this, SLOT(Dummy()), drawTools, 
 		       "draw ring" );
-  drawRing->setPopup(BuildNewRingMenu());
-  drawRing->setPopupDelay(150);
+  ringmenu = BuildNewRingMenu();
+  drawRing->setPopup(ringmenu);
+  drawRing->setPopupDelay(1);
   QWhatsThis::add(drawRing, ringToolText);
 
   symbolIcon = QPixmap( symboltool );
@@ -624,14 +629,6 @@ ApplicationWindow::ApplicationWindow()
   file->setWhatsThis( id, fileSaveText );
   id = file->insertItem( tr("Save picture..."), this, SLOT(savePicture()) );
 
-  //#ifdef UNIX
-  //file->insertSeparator();
-  //id = file->insertItem( tr("Import via OpenBabel"),
-  //		 this, SLOT(OBImport()) );
-  //id = file->insertItem( tr("Export via OpenBabel"),
-  //		 this, SLOT(OBExport()) );
-  //#endif
-
   file->insertSeparator();
   id = file->insertItem( tr("Pa&ge setup"), this , SLOT(PageSetup()) );
   id = file->insertItem( printIcon, tr("&Print"),
@@ -669,10 +666,13 @@ ApplicationWindow::ApplicationWindow()
   id = zoomSub->insertItem(tr("200%"),this,SLOT(Dummy()),CTRL+Key_9);
   */
 
-  QPopupMenu * edit = new QPopupMenu(this);
+  edit = new QPopupMenu(this);
   menuBar()->insertItem(tr("&Edit"), edit);
 
   edit->insertItem(tr("&Undo"), this, SLOT(Undo()), CTRL+Key_Z);
+  insertsymbol_id = edit->insertItem( tr("Insert s&ymbol"), this,
+				      SLOT(InsertSymbol()) );
+  edit->setItemVisible(insertsymbol_id, false);
   edit->insertSeparator();
   edit->insertItem(tr("Cu&t"), this, SLOT(Cut()), CTRL+Key_X);
   edit->insertItem(tr("&Copy"), this, SLOT(Copy()), CTRL+Key_C);
@@ -718,47 +718,41 @@ ApplicationWindow::ApplicationWindow()
   format->insertSeparator();
   format->insertItem(tr("&Drawing settings..."), this, 
 		     SLOT(ShowFixedDialog()));
+  format->insertItem(tr("&XDC settings..."), this, SLOT(XDCSettings()) );
 
   QPopupMenu * reactionTools = new QPopupMenu(this);
   //reactionTools->insertItem( tr("Test..."), this, 
   //		     SLOT(reactionAnalysisTest()) );
   reactionTools->insertItem( tr("Estimate gas-phase enthalphy change"),
 			     this, SLOT(reactionAnalysisEnthalpy()) );
-  reactionTools->insertItem( tr("Estimate partial charges"),
-			     this, SLOT(CalcPartialCharge()) );
   reactionTools->insertItem( tr("Compare 1H NMR"),
 			     this, SLOT(reactionAnalysis1HNMR()) );
   reactionTools->insertItem( tr("Compare 13C NMR"),
 			     this, SLOT(reactionAnalysis13CNMR()) );
+  reactionTools->insertSeparator();
+  //reactionTools->insertItem( tr("Estimate partial charges"),
+  //		     this, SLOT(reactivityPC()) );
+  //reactionTools->insertItem( tr("Forward reactions"),
+  //		     this, SLOT(reactivityForward()) );
+  reactionTools->insertItem( tr("Reverse reactions"),
+			     this, SLOT(reactivityRetro()) );
 
+  /*
   reactionTools->insertItem( tr("Retrosynthesis"), this, SLOT(Retro()) );
+  */
   reactionTools->insertItem( tr("Get bond identifier"), this, 
 			     SLOT(RetroBondName()) );
 
   QPopupMenu * tools = new QPopupMenu(this);
   menuBar()->insertItem(tr("T&ools"), tools);
-  tools->insertItem( tr("Clean up molecule"), this, SLOT(CleanUpMolecule()) );
+  tools->insertItem(tr("Clean up molecule"), this, SLOT(CleanUpMolecule()) );
   tools->insertItem(tr("Auto &layout"), this, SLOT(AutoLayout()), CTRL+Key_L );
   tools->insertItem(tr("Create custom ring"), this,
 		    SLOT(saveCustomRing()) );
   tools->insertSeparator();
   tools->insertItem(tr("Molecule information..."),this,SLOT(MoleculeInfo()) );
-  //tools->insertItem(tr("Calculate empirical formula"),this, SLOT(CalcEF()) );
-  //tools->insertItem(tr("Calculate molecular weight"), this, SLOT(CalcMW()) );
-  tools->insertItem(tr("Elemental analysis"), this, SLOT(CalcEA()) );
 
-  /*
-  FILE *pinfile = popen("python -c \"from frowns import Smiles;mol = Smiles.smilin(\\\"c1ccccc1\\\");print mol.cansmiles()\"", "r");
-  if (pinfile) {
-    char csmiles[256];
-    fgets(csmiles, 250, pinfile);
-    if (strncmp(csmiles, "c1ccccc1", 8) == 0)
-      tools->insertItem("Molecule name", this, SLOT(CalcName()) );
-    pclose(pinfile);
-  }
-  */
-
-  tools->insertItem("Predict 1H NMR", this, SLOT(Calc1HNMR()) );
+  tools->insertItem(tr("Predict 1H NMR"), this, SLOT(Calc1HNMR()) );
   tools->insertItem(tr("Predict 13C NMR"), this, SLOT(Calc13CNMR()) );
   tools->insertItem(tr("Predict IR"), this, SLOT(CalcIR()) );
   tools->insertItem(tr("Predict pKa"), this, SLOT(CalcpKa()) );
@@ -767,7 +761,8 @@ ApplicationWindow::ApplicationWindow()
   tools->insertSeparator();
   tools->insertItem(tr("Input SMILES"), this, SLOT(FromSMILES()) );
   tools->insertItem(tr("Output SMILES"), this, SLOT(ToSMILES()) );
-  //tools->insertItem(tr("Convert to 3-D"), this, SLOT(To3D()) );
+  tools->insertSeparator();
+  tools->insertItem(tr("Build 3D model of molecule"), this, SLOT(To3D()) );
   //tools->insertSeparator();
   //tools->insertItem(tr("Test tool"), this, SLOT(Test()) );
 
@@ -803,6 +798,8 @@ ApplicationWindow::ApplicationWindow()
 	   this, SLOT(HelpTopic(QString)) );
   connect( c, SIGNAL(SignalHelpTopic(QString)),
 	   this, SLOT(HelpTopic(QString)) );
+  connect( c, SIGNAL( SignalUpdateCustomRingMenu() ),
+	   this, SLOT( updateCustomRingMenu() ) );  
 
   //setCentralWidget( sv );
   setCentralWidget( cw );
@@ -810,6 +807,7 @@ ApplicationWindow::ApplicationWindow()
   resize( 640, 640 );
 }
 
+/* obsolete.  removed.
 QPopupMenu *ApplicationWindow::BuildRingMenu() {
   QPopupMenu * ringSub = new QPopupMenu(this);
   QDir d(RingDir, "*.png");
@@ -820,6 +818,7 @@ QPopupMenu *ApplicationWindow::BuildRingMenu() {
   }
   return ringSub;
 }
+*/
 
 QPopupMenu *ApplicationWindow::BuildSymbolMenu() {
   QPopupMenu * symbolSub = new QPopupMenu(this);
@@ -867,21 +866,27 @@ QPopupMenu *ApplicationWindow::BuildArrowMenu() {
   p1 = QPixmap(arrow_regular_xpm);
   arrowlist.append(QString("REGULAR"));
   arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 0);
+  p1 = QPixmap(arrow_topharpoon_xpm);
+  arrowlist.append(QString("TOP_HARPOON"));
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 1);
+  p1 = QPixmap(arrow_bottomharpoon_xpm);
+  arrowlist.append(QString("BOTTOM_HARPOON"));
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 2);
   p1 = QPixmap(arrow_didnt_work_xpm);
   arrowlist.append(QString("DIDNT_WORK"));
-  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 1);
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 3);
   p1 = QPixmap(arrow_dash_xpm);
   arrowlist.append(QString("DASH"));
-  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 3);
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 4);
   p1 = QPixmap(arrow_bi1_xpm);
   arrowlist.append(QString("BI1"));
-  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 4);
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 5);
   p1 = QPixmap(arrow_bi2_xpm);
   arrowlist.append(QString("BI2"));
-  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 5);
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 6);
   p1 = QPixmap(arrow_retro_xpm);
   arrowlist.append(QString("RETRO"));
-  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 6);
+  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 7);
   return arrowSub;
 }
 
@@ -945,16 +950,23 @@ QPopupMenu *ApplicationWindow::BuildBracketMenu() {
   return bracketSub;
 }
 
+// loads custom rings.
 void ApplicationWindow::FromRingMenu(int x) {
   c->StartUndo(0,0);
   c->DeselectAll();
   c->SetTopLeft(sv->viewportToContents(QPoint(0,0)));
   QString fname( ringlist[x] );
   fname.replace(fname.length() - 3, 3, QString("cml"));
-  c->load( RingDir + fname );
+  c->load( preferences.getCustomRingDir() + fname );
   r->Inserted();
 }
 
+void ApplicationWindow::updateCustomRingMenu() {
+  ringmenu->removeItem(customRingMenuId);
+  customRingMenuId = ringmenu->insertItem( tr("User-defined"), 
+					   BuildCustomRingMenu() );
+}
+
 void ApplicationWindow::FromRingToolbar(QString fi) {
   c->DeselectAll();
   if (fi.contains("6ring") > 0) {
@@ -1043,6 +1055,7 @@ void ApplicationWindow::SetThick(int t) {
 }
 
 void ApplicationWindow::ShowTextButtons(QFont infont) {
+  edit->setItemVisible(insertsymbol_id, true);
   leftJustify->show();
   centerJustify->show();
   rightJustify->show();
@@ -1068,6 +1081,7 @@ void ApplicationWindow::ShowTextButtons(QFont infont) {
 }
 
 void ApplicationWindow::HideTextButtons() {
+  edit->setItemVisible(insertsymbol_id, false);
   leftJustify->hide();
   centerJustify->hide();
   rightJustify->hide();
@@ -1228,7 +1242,7 @@ void ApplicationWindow::saveAs()
 void ApplicationWindow::savePicture()
 {
   int pm = 0;
-  bool was_saved;
+  bool was_saved = false;
   QString picfile;
 
   MyFileDialog fd(0, QString::null);
@@ -1301,6 +1315,8 @@ void ApplicationWindow::SavePNG() {
 }
 
 void ApplicationWindow::Save3D() {
+  bool was_saved;
+  c->Save3D( ni_savefile );
   exit(0);  // this function is only used in non-interactive mode
 }
 
@@ -1373,6 +1389,7 @@ void ApplicationWindow::SetStatusBar(QString s) {
   statusBar()->message(s);
 }
 
+/* obsolete.
 void ApplicationWindow::MakeRingDialog() {
   RingDialog t(this, "ring dialog");
   if ( !t.exec() ) return;
@@ -1383,9 +1400,9 @@ void ApplicationWindow::MakeRingDialog() {
   c->load( RingDir + t.getFile() );
   r->Inserted();
 }
+*/
 
 void ApplicationWindow::ShowFixedDialog(void) {
-  double na, nl;
   FixedDialog i(this, QString("fixed_dialog"));
   if ( !i.exec() ) return;
   hruler->repaint();
@@ -1523,7 +1540,7 @@ void ApplicationWindow::HelpTopic(QString topic) {
 
 // new manual (based on QTextBrowser) -- derived from Qt example "helpviewer"
 /****************************************************************************
-** $Id: application.cpp,v 1.18 2004/06/08 20:39:32 bherger Exp $
+** $Id: application.cpp,v 1.34 2005/03/22 05:42:53 bherger Exp $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -1563,9 +1580,6 @@ void ApplicationWindow::Calc13CNMR() { r->Tool(MODE_TOOL_13CNMR); }
 void ApplicationWindow::Calc1HNMR() { r->Tool(MODE_TOOL_1HNMR); }
 void ApplicationWindow::CalcIR() { r->Tool(MODE_TOOL_IR); }
 void ApplicationWindow::CalcpKa() { r->Tool(MODE_TOOL_PKA); }
-void ApplicationWindow::CalcPartialCharge() { 
-  //r->Tool(MODE_TOOL_PARTIAL_CHARGE); 
-}
 void ApplicationWindow::CalcKOW() { r->Tool(MODE_TOOL_KOW); }
 void ApplicationWindow::CalcName() { r->Tool(MODE_TOOL_NAME); }
 void ApplicationWindow::ToSMILES() { r->Tool(MODE_TOOL_TOSMILES); }
@@ -1599,11 +1613,19 @@ void ApplicationWindow::clearGroup() {
 void ApplicationWindow::Retro() {
   r->Tool(MODE_TOOL_RETRO);
 }
+void ApplicationWindow::RetroAtomName() {
+  r->Tool(MODE_TOOL_RETRO_ATOMNAME);
+}
 void ApplicationWindow::RetroBondName() {
   r->Tool(MODE_TOOL_RETRO_BONDNAME);
 }
+void ApplicationWindow::InsertSymbol() {
+  r->InsertSymbol();
+}
 void ApplicationWindow::Test() {
-  r->Tool(MODE_TOOL_TEST);
+  CharSelDialog cd1(this, "charsel");
+  cd1.exec();
+  //r->Tool(MODE_TOOL_TEST);
 }
 void ApplicationWindow::clearAllGroups() { 
   r->clearAllGroups();
@@ -1620,6 +1642,16 @@ void ApplicationWindow::reactionAnalysis1HNMR() {
 void ApplicationWindow::reactionAnalysis13CNMR() {
   r->ReactionAnalysis(RXN_13CNMR);
 }
+void ApplicationWindow::reactivityForward() {
+  r->Tool(MODE_TOOL_REACTIVITY_FORWARD);
+}
+void ApplicationWindow::reactivityRetro() {
+  r->Tool(MODE_TOOL_REACTIVITY_RETRO);
+}
+void ApplicationWindow::reactivityPC() {
+  r->Tool(MODE_TOOL_CHARGES);
+}
+
 void ApplicationWindow::toggleGrid() {
   int i1;
   i1 = preferences.getDrawGrid();
@@ -1668,3 +1700,37 @@ void ApplicationWindow::Zoom(int z1) {
   vruler->zoomEvent();
   SetStatusBar( tr("Zoom = %1 %").arg(preferences.getZoom()) );
 }
+void ApplicationWindow::XDCSettings() {
+  bool ok; QFont font;
+  switch( QMessageBox::information( this, tr("XDC Settings"),
+				    tr("Change XDrawChem settings:"),
+				    tr("&Main font"), tr("&Ruler font"), 
+				    tr("Cancel"),
+				    2,      // Enter == button 2
+				    2 ) ) { // Escape == button 2
+  case 0: 
+    font = QFontDialog::getFont(&ok, preferences.getMainFont(), this );
+    if ( ok ) {
+      preferences.setMainFont(font);
+      setFont(font);
+      repaint();
+    } else {
+      return;
+    }
+    break;
+  case 1: 
+    font = QFontDialog::getFont(&ok, preferences.getRulerFont(), this );
+    if ( ok ) {
+      preferences.setRulerFont(font);
+      //setFont(font);
+      hruler->repaint(); vruler->repaint();
+      repaint();
+    } else {
+      return;
+    }
+    break;
+  case 2: 
+    return;
+    break;
+  }
+}
