@@ -210,6 +210,12 @@ void ApplicationWindow::OBNewSave() {
   strcpy(realchar, tmpchar);
   inFileType = extab.FilenameToType( realchar );
   OBMol *mol = new OBMol( inFileType, inFileType );
+  
+  // invert so save file has correct coordinates
+  r->SelectAll();
+  r->Flip(FLIP_V);
+  r->DeselectAll();
+
   IOIface converter(c, mol);
   if ( !converter.convertToOBMol() ) {
     QMessageBox::warning( 0, tr("IOIface Data error"),
