@@ -344,11 +344,18 @@ QPopupMenu *ApplicationWindow::BuildNewRingMenu() {
   ringSub->insertItem( tr("Useful groups"), fgSub );
 
   // make user-defined list
+  customRingMenuId = ringSub->insertItem( tr("User-defined"), 
+					  BuildCustomRingMenu() );
+
+  return ringSub;
+}
+
+QPopupMenu *ApplicationWindow::BuildCustomRingMenu() {
   QPopupMenu * userDefSub = new QPopupMenu(this);
   QDir d(preferences.getCustomRingDir(), "*.png");
   ringlist = d.entryList();
   for (int cc = 0; cc < ringlist.count(); cc++) {
-    QPixmap px1( QString(RingDir + ringlist[cc]) );
+    QPixmap px1( QString(preferences.getCustomRingDir() + ringlist[cc]) );
     QBitmap mask1( px1.width(), px1.height() );
     mask1.fill(Qt::color1);
     px1.setMask(mask1);
@@ -359,10 +366,9 @@ QPopupMenu *ApplicationWindow::BuildNewRingMenu() {
 			    ringlist[cc].left(ringlist[cc].length() - 4),
 			    this, SLOT(FromRingMenu(int)), 0, cc );
   }
-  //userDefSub->insertItem( tr("Add new...") );
-  ringSub->insertItem( tr("User-defined"), userDefSub );
+  userDefSub->insertItem( tr("Add new..."), this, SLOT( saveCustomRing() ) );
 
-  return ringSub;
+  return userDefSub;
 }
 
 void ApplicationWindow::FromNewRingMenu(int x) {
