@@ -8,7 +8,6 @@ using std::cout;
 using std::endl;
 
 #include "netaccess.h"
-#include "molinfodialog.h"
 #include "crings_dialog.h"
 #include "tooldialog.h"
 #include "tool_1h_nmr.h"
@@ -21,6 +20,7 @@ using std::endl;
 #include "arrow.h"
 #include "bracket.h"
 #include "text.h"
+#include "gobject.h"
 #include "chemdata.h"
 #include "defs.h"
 
@@ -66,18 +66,18 @@ public:
 
 // Determine Molecule clicked, do Tool action
 void ChemData::Tool(DPoint *target, int mode) {
+  std::cout << "ChemData::Tool: " << mode << std::endl;
   Molecule *m = 0;
-  Text *tt, *tt_mw, *tt_ef, *tt_ea;
-  MolInfoDialog mi(r, "mol info");
   NetAccess na;
   CustomRingDialog cr1;
   ToolDialog tool1;
-  Tool_1HNMR_Dialog tool1hnmr;
-  Tool_13CNMR_Dialog tool13cnmr;
-  Tool_IR_Dialog toolir;
+  Tool_1HNMR_Dialog *tool1hnmr;
+  Tool_13CNMR_Dialog *tool13cnmr;
+  Tool_IR_Dialog *toolir;
   Tool_2D3D tool2d3d;
   QString tmpname, serverName;
   QStringList choices;
+  Bond *tmp_bond;
 
   for (tmp_draw = drawlist.first(); tmp_draw!=0; tmp_draw = drawlist.next()) {
     if (tmp_draw->Type() == TYPE_MOLECULE) {
@@ -90,24 +90,29 @@ void ChemData::Tool(DPoint *target, int mode) {
   double kow = 0.0;
   switch (mode) {
   case MODE_TOOL_MOLECULE_INFO:
+    mi = new MolInfoDialog(r, "mol info");
     tt_mw = m->CalcMW();
     tt_ef = m->CalcEmpiricalFormula();
-    tt_ea = m->CalcElementalAnalysis(false);
-    mi.setMW(tt_mw->getText());
-    mi.setEF(tt_ef->getText());
-    mi.setEA(tt_ea->getText());
+    tt_ea = m->CalcElementalAnalysis();
+    mi->setMW(tt_mw->getText());
+    mi->setEF(tt_ef->getText());
+    mi->setEA(tt_ea->getText());
 
     serverName = getenv("XDC_SERVER");
     if (serverName.length() < 2)
       serverName = XDC_SERVER;
     if (na.getNameCAS(serverName, m->CalcName())) {
-      mi.setCAS(na.scas);
-      mi.setName(na.sname);
+      mi->setCAS(na.scas);
+      mi->setName(na.sname);
     }
 
-    if ( !mi.exec() ) return;    
-    if (mi.isMWChecked()) drawlist.append(tt_mw);
-    if (mi.isEFChecked()) drawlist.append(tt_ef);
+    connect( mi, SIGNAL(MIDClose()), this, SLOT(returnFromMID()) );
+
+    mi->show();
+    //if ( !mi->exec() ) return;    
+    //if (mi->isMWChecked()) drawlist.append(tt_mw);
+    //if (mi->isEFChecked()) drawlist.append(tt_ef);
+    //delete mi;
     break;
   case MODE_TOOL_CALCMW:
     tt = m->CalcMW();
@@ -122,18 +127,21 @@ void ChemData::Tool(DPoint *target, int mode) {
     if (tt != 0) drawlist.append(tt);
     break;
   case MODE_TOOL_13CNMR:
-    tool13cnmr.setMolecule(m);
-    tool13cnmr.exec();
+    tool13cnmr = new Tool_13CNMR_Dialog;
+    tool13cnmr->setMolecule(m);
+    tool13cnmr->show();
     //m->Calc13CNMR();
     break;
   case MODE_TOOL_1HNMR:
-    tool1hnmr.setMolecule(m);
-    tool1hnmr.exec();
+    tool1hnmr = new Tool_1HNMR_Dialog;
+    tool1hnmr->setMolecule(m);
+    tool1hnmr->show();
     //m->Calc1HNMR();
     break;
   case MODE_TOOL_IR:
-    toolir.setMolecule(m);
-    toolir.exec();
+    toolir = new Tool_IR_Dialog;
+    toolir->setMolecule(m);
+    toolir->show();
     //m->CalcIR();
     break;
   case MODE_TOOL_PKA:
@@ -142,19 +150,40 @@ void ChemData::Tool(DPoint *target, int mode) {
   case MODE_TOOL_RETRO:
     Retro(m);
     break;
+  case MODE_TOOL_REACTIVITY_FORWARD:
+    m->Reactivity(mode);
+    break;
+  case MODE_TOOL_REACTIVITY_RETRO:
+    m->Reactivity(mode);
+    for (tmp_bond = m->bondsFirst(); tmp_bond != NULL;
+	 tmp_bond = m->bondsNext()) {
+      if (tmp_bond->getReactions().length() > 2) {
+	tmp_bond->SetColor( QColor(124,252,0) );
+      }
+    }
+    r->repaint();
+    break;
+  case MODE_TOOL_CHARGES:
+    m->Reactivity(mode);
+    break;
   case MODE_TOOL_KOW:
     kow = m->CalcKOW();
     QMessageBox::information(r, tr("Octanol-water partition"), tr("Estimated octanol-water partition constant (log Kow) = %1").arg(kow) );
     break;
   case MODE_TOOL_2D3D:
-    tool2d3d.setMolecule(m);
+    //tool2d3d.setMolecule(m);
+    //na.get3DMol( m->ToMDLMolfile() );
+    m->Make3DVersion();
+    //std::cout << m->ToMDLMolfile() << std::endl;
+    //std::cout << na.s3dmol << std::endl;
     break;
   case MODE_TOOL_NAME:
     m->CalcName();
     break;
   case MODE_TOOL_CUSTOMRING:
     cr1.setMolecule(m);
-    cr1.exec();
+    if (cr1.exec() == QDialog::Accepted)
+      emit SignalUpdateCustomRingMenu();
     break;
   case MODE_TOOL_TOSMILES:
     tmpname = m->ToSMILES();
@@ -183,6 +212,36 @@ void ChemData::Tool(DPoint *target, int mode) {
   // Need to pick next tool manually
 }
 
+void ChemData::Save3D(QString fn3d) {
+  // save 3D image of first molecule
+  Molecule *m = 0;
+
+  for (tmp_draw = drawlist.first(); tmp_draw!=0; tmp_draw = drawlist.next()) {
+    if (tmp_draw->Type() == TYPE_MOLECULE) {
+      m = (Molecule *)tmp_draw;
+      break;
+    }
+  }
+
+  if (m != 0) {
+    m->Make3DVersion(fn3d);
+  } else {
+    std::cout << "No molecule in input file!" << std::endl;
+  }
+}
+
+void ChemData::returnFromMID() {
+  Q_CHECK_PTR(mi);
+  Q_CHECK_PTR(tt_mw);
+  Q_CHECK_PTR(tt_ef);
+  Q_CHECK_PTR(tt_ea);
+  if (mi->isMWChecked()) drawlist.append(tt_mw);
+  if (mi->isEFChecked()) drawlist.append(tt_ef);
+  if (mi->isEAChecked()) drawlist.append(tt_ea);
+  mi->hide();
+  delete mi;
+}
+
 void ChemData::clearAllGroups() {
   Molecule *m = 0;
   QString tmpname;
@@ -405,6 +464,13 @@ void ChemData::SmartPlace(QString sf, DPoint *t1) {
   load(sf);
   tmp_draw = drawlist.last();
   Molecule *m1 = (Molecule *)tmp_draw;
+  if (fabs(ang1) < 0.1) {
+    ang1 = 3.14159;
+  } else {
+    if (fabs(ang1) > 3.13) {
+      ang1 = 0.0;
+    }
+  }
   cout << "angle = " << (ang1 * 180.0 / 3.14159) << " degrees!" << endl;
   tmp_pt = m1->GetRingAttachPoint();
   m1->Rotate(ang1);
@@ -482,7 +548,8 @@ double ChemData::CalculateRingAttachAngle(DPoint *t1) {
   Molecule *m = 0;
 
   for (tmp_draw = drawlist.first(); tmp_draw!=0; tmp_draw = drawlist.next()) {
-    if (tmp_draw->Type() == TYPE_MOLECULE) {
+    if ( (tmp_draw->Type() == TYPE_MOLECULE) &&
+	 (tmp_draw->Find(t1) == true) ) {
       m = (Molecule *)tmp_draw;
       break;
     }
@@ -501,7 +568,7 @@ bool ChemData::SelectWithinLoop(QPointArray tmp_lasso) {
 
   int side_up, side_down, side_left, side_right;
   bool retval;
-  double xmin, xmax, ymin, ymax, tmp_swap, xcenter, ycenter;
+  double xmin, xmax, ymin, ymax, tmp_swap; // xcenter, ycenter;
   int lcount;
   QPoint tmp_bb;
   DPoint *t1, *t2;
