@@ -3,20 +3,20 @@
 
 #include <qstring.h>
 
-// Ring directory
-extern QString RingDir;
+// Ring directories (global, user)
+extern QString RingDir, HomeDir;
 
 // version string (makes updates easier :)
 #ifdef UNIX
-#define XDC_VERSION "XDrawChem 1.8.5"
+#define XDC_VERSION "XDrawChem 1.9.4"
 #endif
 
 #ifdef WIN32
-#define XDC_VERSION "WinDrawChem 1.8.5"
+#define XDC_VERSION "WinDrawChem 1.9.4"
 #endif
 
 #ifdef MACOSX
-#define XDC_VERSION "MacDrawChem 1.8.5"
+#define XDC_VERSION "MacDrawChem 1.9.4"
 #endif
 
 // define XDC_SERVER for Windows
@@ -116,10 +116,18 @@ int my_round(double);
 #define MODE_TOOL_GROUP_CLEAR 620
 #define MODE_TOOL_2D3D 630
 #define MODE_TOOL_PKA 640
-#define MODE_TOOL_PARTIAL_CHARGE 641
 #define MODE_TOOL_KOW 650
 #define MODE_TOOL_RETRO 660
+#define MODE_TOOL_RETRO_ATOMNAME 698
 #define MODE_TOOL_RETRO_BONDNAME 699
+
+// list (unique?) reactions/sites this molecule is susceptible to
+#define MODE_TOOL_REACTIVITY_FORWARD 670
+// list (unique?) sites that could have been created by known rxn's
+#define MODE_TOOL_REACTIVITY_RETRO 671
+// partial charges, maybe do colors, maybe base on AMBER or other FF
+#define MODE_TOOL_CHARGES 672
+
 #define MODE_TOOL_TEST 669
 
 // reaction analysis
@@ -139,6 +147,9 @@ int my_round(double);
 #define TYPE_CURVEARROW 15
 #define TYPE_BEZIER 16
 #define TYPE_GRAPHIC_OBJECT 99
+#define TYPE_GRAPHIC_LINE 100
+#define TYPE_GRAPHIC_ATOMMARK 101
+#define TYPE_GRAPHIC_BONDMARK 102
 
 // bond stereochemistry (Bond::stereo)
 #define STEREO_UNDEFINED 0
@@ -167,6 +178,8 @@ int my_round(double);
 #define ARROW_BI1 3
 #define ARROW_BI2 4
 #define ARROW_RETRO 5
+#define ARROW_TOPHARPOON 6
+#define ARROW_BOTTOMHARPOON 7
 
 // curve arrow styles
 #define CA_CW90 1
