@@ -9,6 +9,7 @@
 #include <qfont.h>
 #include <qtextstream.h>
 #include <qlist.h>
+#include <qregexp.h>
 #include <iostream>
 
 using std::cout;
@@ -20,6 +21,7 @@ class DPoint {
  public:
   double x, y, z;
   double x3, y3, z3;
+  double dx, dy, dz;  // where to go on this iteration of (3D/minimize)
   QString id; // XML ID field
   QString element;  // OR text string if associated with Text
   QString elementmask; // text mask for text string
@@ -46,6 +48,9 @@ class DPoint {
   QList<DPoint> path;
   // for reconstructing rings...
   int new_order;
+  // reactivity
+  double partial_charge;  // + or -, should also set "color" above
+  QString react_atom_type;  // AMBER atom type?  Custom?
   // spectral data
   bool nmr_proton;  // is this DPoint a proton added for 1H NMR prediction?
   double H1_shift;
@@ -123,6 +128,11 @@ class DPoint {
     x = (double)p.x();
     y = (double)p.y();
   }
+  void set(double xn, double yn, double zn) {
+    x3 = xn;
+    y3 = yn;
+    z3 = zn;
+  }
   QPoint toQPoint() {
     int x1 = (int)x;
     int y1 = (int)y;
@@ -145,6 +155,32 @@ class DPoint {
     double dy = y - t->y;
     return (double)sqrt(dx * dx + dy * dy);
   }
+  double Magnitude() {
+    return sqrt( (x3*x3) + (y3*y3) + (z3*z3) );
+  }
+  void Normalize() {
+    double dst = Magnitude();
+    x3 /= dst;
+    y3 /= dst;
+    z3 /= dst;
+  }
+  void Scale(double sf) {
+    double dst = Magnitude();
+    Normalize();
+    dst = dst * sf;
+    x3 *= dst;
+    y3 *= dst;
+    z3 *= dst;
+  }
+  void SetMagnitude(double sf) {
+    double dst = Magnitude();
+    sf = sf / dst;
+    Normalize();
+    dst = dst * sf;
+    x3 *= dst;
+    y3 *= dst;
+    z3 *= dst;
+  }
   void SetFontFromXML(QString xml_tag) {
     cout << "DP::SetFontFromXML:" << xml_tag << endl;
     int i1, i2;
@@ -247,6 +283,8 @@ class DPoint {
     i2 = xml_tag.find("</symtype>") + 10;
     if (i1 >= 0)
       SetSymbolFromXML(xml_tag.mid(i1, i2 - i1));
+    i1 = xml_tag.find("<textobject>yes</textobject>");
+    if (i1 >= 0) hit = true;  // Create Text object associated with this DPoint
   }
   // return atomic number of atom connected in this group
   int getAtomicNumber() {
@@ -297,6 +335,15 @@ class DPoint {
     if (tc1 == 17) return "Cl";
     if (tc1 == 35) return "Br";
     if (tc1 == 53) return "I";
+    if (tc1 == 999) {
+      // let's try this...
+      QString bele = element;
+      bele.remove("+");
+      bele.remove("-");
+      bele.remove("H");
+      bele.remove( QRegExp("\\d+") );
+      return bele;
+    }
     return "X";
   }
 };
