@@ -96,7 +96,7 @@ void Molecule::Render(void)
     tmp_bond->Render();
   for (tmp_text = labels.first(); tmp_text != NULL; tmp_text = labels.next()) {
     // ignore if just "C"
-    if (tmp_text->getText() == "C") continue;
+    //if (tmp_text->getText() == "C") continue;
     tmp_text->Render();
   }
   CalcOffsets();
@@ -502,6 +502,7 @@ DPoint * Molecule::FindNearestPoint(DPoint *target, double &dist) {
     if (d1dist < mindist) { mindist = d1dist; nearest = d1; }
   }
   dist = mindist;
+  if (!nearest) return 0;
   nearest->other = 0;
   bool already_found = false;
   for (tmp_bond = bonds.first(); tmp_bond != NULL; tmp_bond = bonds.next()) {
@@ -728,12 +729,14 @@ QString Molecule::ToXML(QString xml_id) {
   // Copy text from Text objects to element field in DPoint
   // also copy Symbol descriptions
   for (tmp_draw = uo.first(); tmp_draw != NULL; tmp_draw = uo.next() ) {
+    tmp_draw->Start()->hit = false;
     if (tmp_draw->Type() == TYPE_TEXT) {
       tmp_text = (Text *)tmp_draw;  // is this cheating?
       tmp_text->Start()->element = tmp_text->getText();
       tmp_text->Start()->elementmask = tmp_text->getTextMask();
       tmp_text->Start()->color = tmp_text->GetColor();
       tmp_text->Start()->font = tmp_text->getFont();
+      tmp_text->Start()->hit = true;
     }
     if (tmp_draw->Type() == TYPE_SYMBOL) {
       tmp_sym = (Symbol *)tmp_draw;
@@ -768,6 +771,7 @@ QString Molecule::ToXML(QString xml_id) {
     s.append("<elemask>");
     s.append(tmp_pt->elementmask);
     s.append("</elemask>\n");
+    if (tmp_pt->hit) s.append("<textobject>yes</textobject>");
     if (tmp_pt->symbol.length() > 1) {
       s.append("<symtype>");
       s.append(tmp_pt->symbol);
@@ -935,7 +939,7 @@ QString Molecule::ToCDXML(QString xml_id) {
 }
 
 // Generate MDL Molfile to export to generate SMILES, export for 3-D 
-QString Molecule::ToMDLMolfile() {
+QString Molecule::ToMDLMolfile(int coords) {
   QList<DPoint> up;
   QList<Drawable> uo;
   Bond *tmp_bond;
@@ -969,10 +973,20 @@ QString Molecule::ToMDLMolfile() {
     tmpline = tmp_pt->element;
     if (tmpline.length() < 3) tmpline.append(' ');
     if (tmpline.length() < 3) tmpline.append(' ');
-    t.width(10);
-    t << tmp_pt->x;
-    t.width(10);
-    t << -tmp_pt->y << "    0.0000 " << tmpline;
+    if (coords == 0) {  // 2D
+      t.width(10);
+      t << tmp_pt->x;
+      t.width(10);
+      t << -tmp_pt->y << "    0.0000 " << tmpline;
+    } else {  // 3D
+      t.width(10);
+      t << tmp_pt->x;
+      t.width(10);
+      t << tmp_pt->y;
+      t.width(10);
+      t << tmp_pt->z;
+      t << tmpline;
+    }
     t << " 0  0  0  0  0  0  0  0  0  0  0  0" << endl;
   }
   // write bond list
@@ -1098,7 +1112,7 @@ void Molecule::FromXML(QString xml_tag) {
   } while (1);
   // add Text and Symbol as needed
   for (tmp_pt = points.first(); tmp_pt != NULL; tmp_pt = points.next()) {
-    if (tmp_pt->element != QString("C")) {
+    if ( (tmp_pt->element != QString("C")) || (tmp_pt->hit == true) ) {
       tmp_text = new Text(r);
       tmp_text->setPoint(tmp_pt);
       tmp_text->SetColor(tmp_pt->color);
