@@ -30,6 +30,8 @@ public:
   // BFS queue
   QList<DPoint> bfs_queue;
 
+  int IsInRing(DPoint *);
+  int InSameRing(DPoint *, DPoint *, DPoint *);
   void PrintSSSR();
   void FindAromatic(QList<Bond>);
   void BuildSSSR(QList<DPoint>);
