@@ -37,6 +37,28 @@ void Molecule::MakeSSSR() {
   this_sssr.FindAromatic(bonds);
 }
 
+int Molecule::OrderOfBond(DPoint *e1, DPoint *e2) {
+  for (tmp_bond = bonds.first(); tmp_bond != 0; tmp_bond = bonds.next()) {
+    if (tmp_bond->Find(e1) == true) {
+      tmp_pt = tmp_bond->otherPoint(e1);
+      if (tmp_pt == e2) return tmp_bond->Order();
+    }  
+  }
+
+  return 0;
+}
+
+Bond * Molecule::FindBond(DPoint *e1, DPoint *e2) {
+  for (tmp_bond = bonds.first(); tmp_bond != 0; tmp_bond = bonds.next()) {
+    if (tmp_bond->Find(e1) == true) {
+      tmp_pt = tmp_bond->otherPoint(e1);
+      if (tmp_pt == e2) return tmp_bond;
+    }  
+  }
+
+  return 0;
+}
+
 QString Molecule::CalcName() {
   QString nom = ToSMILES();
   nom = nom.stripWhiteSpace();
@@ -146,7 +168,7 @@ QStringList Molecule::Calc13CNMR(bool show_dialog) {
     }
   }
 
-  GraphDialog *g = new GraphDialog(r, "Predicted 13C-NMR");
+  GraphDialog *g = new GraphDialog(r, tr("Predicted 13C-NMR"));
   // print (for now) list of NMR values
   for ( QStringList::Iterator it = hosecodes.begin(); 
 	it != hosecodes.end(); ++it ) {
@@ -209,7 +231,7 @@ void Molecule::CalcIR() {
 
   peaklist.clear();
 
-  GraphDialog *g = new GraphDialog(r, "Predicted IR");
+  GraphDialog *g = new GraphDialog(r, tr("Predicted IR"));
   // iterate thru all Bonds in Molecule
   QListIterator<Bond> outer(bonds), inner(bonds);
   for ( ; outer.current(); ++outer) {
@@ -223,66 +245,66 @@ void Molecule::CalcIR() {
     if (atom1 > atom2) { swp = atom1; atom1 = atom2; atom2 = swp; }
     if (lorder == 1) {
       if ( (atom1 == 6) && (atom2 == 7) )
-	AddPeak(1300.0, QString("C-N"), QString("~1350-1000, C-N"));
+	AddPeak(1300.0, QString("C-N"), QString(tr("~1350-1000, C-N")));
       if ( (atom1 == 6) && (atom2 == 8) )
-	AddPeak(1300.0, QString("C-O"), QString("~1300-1000, C-O"));
+	AddPeak(1300.0, QString("C-O"), QString(tr("~1300-1000, C-O")));
     }
     if (lorder == 2) {
       if ( (atom1 == 6) && (atom2 == 6) )
-	AddPeak(1660.0, QString("C=C"), QString("~1660-1600, C=C (cis/vinyl strong; trans weak)"));
+	AddPeak(1660.0, QString("C=C"), QString(tr("~1660-1600, C=C (cis/vinyl strong; trans weak)")));
       if ( (atom1 == 6) && (atom2 == 7) )
-	AddPeak(1670.0, QString("C=N"), QString("~1690-1640, C=N"));
+	AddPeak(1670.0, QString("C=N"), QString(tr("~1690-1640, C=N")));
       if ( (atom1 == 6) && (atom2 == 8) )
-	AddPeak(1700.0, QString("C=O"), QString("~1700 (narrow), C=O"));
+	AddPeak(1700.0, QString("C=O"), QString(tr("~1700 (narrow), C=O")));
       if ( (atom1 == 8) && (atom2 == 16) )
-	AddPeak(1350.0, QString("S=O"), QString("~1350-1300, S=O (~1050 if R-(S=O)-R')"));
+	AddPeak(1350.0, QString("S=O"), QString(tr("~1350-1300, S=O (~1050 if R-(S=O)-R')")));
     }
     if (lorder == 3) {
       if ( (atom1 == 6) && (atom2 == 7) )
-	AddPeak(2250.0, QString("CN"), QString("~2250 (narrow), nitrile"));
+	AddPeak(2250.0, QString("CN"), QString(tr("~2250 (narrow), nitrile")));
     }
 
     /* ####################################################################
     if (tmp_bond->Order() == 1) {
       if ( (tmp_bond->Start()->element == "C") &&
 	   (tmp_bond->End()->element.contains("O") > 0) ) {
-	AddPeak(1300.0, QString("C-O"), QString("~1300-1000, C-O"));
+	AddPeak(1300.0, QString("C-O"), QString(tr("~1300-1000, C-O")));
       }
       if ( (tmp_bond->Start()->element.contains("O") > 0) &&
 	   (tmp_bond->End()->element == "C") ) {
-	AddPeak(1300.0, QString("C-O"), QString("~1300-1000, C-O"));
+	AddPeak(1300.0, QString("C-O"), QString(tr("~1300-1000, C-O")));
       }
       if ( (tmp_bond->Start()->element == "C") &&
 	   (tmp_bond->End()->element == "N") ) {
-	AddPeak(1300.0, QString("C-N"), QString("~1350-1000, C-N"));
+	AddPeak(1300.0, QString("C-N"), QString(tr("~1350-1000, C-N")));
       }
       if ( (tmp_bond->Start()->element == "N") &&
 	   (tmp_bond->End()->element == "C") ) {
-	AddPeak(1300.0, QString("C-N"), QString("~1350-1000, C-N"));
+	AddPeak(1300.0, QString("C-N"), QString(tr("~1350-1000, C-N")));
       }      
     }
     if (tmp_bond->Order() == 2) {
       if ( (tmp_bond->Start()->element == "C") &&
 	   (tmp_bond->End()->element == "O") ) {  // ketone
-	AddPeak(1700.0, QString("C=O"), QString("~1700 (narrow), ketone"));
+	AddPeak(1700.0, QString("C=O"), QString(tr("~1700 (narrow), ketone")));
       }
       if ( (tmp_bond->Start()->element == "O") &&
 	   (tmp_bond->End()->element == "C") ) {  // ketone
-	AddPeak(1700.0, QString("C=O"), QString("~1700 (narrow), ketone"));
+	AddPeak(1700.0, QString("C=O"), QString(tr("~1700 (narrow), ketone")));
       }
       if ( (tmp_bond->Start()->element == "C") &&
 	   (tmp_bond->End()->element == "C") ) {  // carbon double bond
-	AddPeak(1700.0, QString("C=C"), QString("~1660-1600, C=C (cis/vinyl strong; trans weak)"));
+	AddPeak(1700.0, QString("C=C"), QString(tr("~1660-1600, C=C (cis/vinyl strong; trans weak)")));
       }
     }
     if (tmp_bond->Order() == 3) {
       if ( (tmp_bond->Start()->element == "C") &&
 	   (tmp_bond->End()->element == "N") ) {  // nitrile
-	AddPeak(2250.0, QString("CN"), QString("~2250 (narrow), nitrile"));
+	AddPeak(2250.0, QString("CN"), QString(tr("~2250 (narrow), nitrile")));
       }
       if ( (tmp_bond->Start()->element == "N") &&
 	   (tmp_bond->End()->element == "C") ) {  // nitrile
-	AddPeak(2250.0, QString("CN"), QString("~2250 (narrow), nitrile"));
+	AddPeak(2250.0, QString("CN"), QString(tr("~2250 (narrow), nitrile")));
       }
     }
     #########################################################################
@@ -292,58 +314,58 @@ void Molecule::CalcIR() {
   for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next()) {
     cout << "|" << tmp_pt->element << "|" << endl;
     if ( (tmp_pt->element == "C") && (tmp_pt->substituents < 4) )
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "CH")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "HC")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "CH2")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "H2C")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "CH3")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if (tmp_pt->element == "H3C")
-      AddPeak(3000.0, QString("CH"), QString("~3000 (broad), C-H"));
+      AddPeak(3000.0, QString("CH"), QString(tr("~3000 (broad), C-H")));
     if ( (tmp_pt->element == "N") && (tmp_pt->substituents == 1) )
-      AddPeak(3350.0, QString("NH"), QString("two peaks: ~3400, ~3300, primary N-H"));
+      AddPeak(3350.0, QString("NH"), QString(tr("two peaks: ~3400, ~3300, primary N-H")));
     if ( (tmp_pt->element == "N") && (tmp_pt->substituents == 2) )
-      AddPeak(3300.0, QString("NH"), QString("~3300 (broad), secondary N-H"));
+      AddPeak(3300.0, QString("NH"), QString(tr("~3300 (broad), secondary N-H")));
     if (tmp_pt->element == "NH")
-      AddPeak(3300.0, QString("NH"), QString("~3300 (broad), secondary N-H"));
+      AddPeak(3300.0, QString("NH"), QString(tr("~3300 (broad), secondary N-H")));
     if (tmp_pt->element == "HN")
-      AddPeak(3300.0, QString("NH"), QString("~3300 (broad), secondary N-H"));
+      AddPeak(3300.0, QString("NH"), QString(tr("~3300 (broad), secondary N-H")));
     if (tmp_pt->element == "NH2")
-      AddPeak(3350.0, QString("NH"), QString("two peaks: ~3400, ~3300, primary N-H"));
+      AddPeak(3350.0, QString("NH"), QString(tr("two peaks: ~3400, ~3300, primary N-H")));
     if (tmp_pt->element == "H2N")
-      AddPeak(3350.0, QString("NH"), QString("two peaks: ~3400, ~3300, primary N-H"));
+      AddPeak(3350.0, QString("NH"), QString(tr("two peaks: ~3400, ~3300, primary N-H")));
     if ( (tmp_pt->element == "S") && (tmp_pt->substituents < 2) )
-      AddPeak(2550.0, QString("SH"), QString("~2550 (broad), S-H"));
+      AddPeak(2550.0, QString("SH"), QString(tr("~2550 (broad), S-H")));
     if ( (tmp_pt->element == "SH") && (tmp_pt->substituents < 2) )
-      AddPeak(2550.0, QString("SH"), QString("~2550 (broad), S-H"));
+      AddPeak(2550.0, QString("SH"), QString(tr("~2550 (broad), S-H")));
     if ( (tmp_pt->element == "HS") && (tmp_pt->substituents < 2) )
-      AddPeak(2550.0, QString("SH"), QString("~2550 (broad), S-H"));
+      AddPeak(2550.0, QString("SH"), QString(tr("~2550 (broad), S-H")));
     if ( (tmp_pt->element == "O") && (tmp_pt->substituents < 2) )
-      AddPeak(2550.0, QString("OH"), QString("~3400 (broad), O-H"));
+      AddPeak(2550.0, QString("OH"), QString(tr("~3400 (broad), O-H")));
     if (tmp_pt->element == "CN")
-      AddPeak(2250.0, QString("CN"), QString("~2250 (narrow), nitrile"));
+      AddPeak(2250.0, QString("CN"), QString(tr("~2250 (narrow), nitrile")));
     if (tmp_pt->element == "NC")
-      AddPeak(2250.0, QString("CN"), QString("~2250 (narrow), nitrile"));
+      AddPeak(2250.0, QString("CN"), QString(tr("~2250 (narrow), nitrile")));
     if (tmp_pt->element == "NCO")
-      AddPeak(2270.0, QString("NCO"), QString("~2270 (narrow), -N=C=O"));
+      AddPeak(2270.0, QString("NCO"), QString(tr("~2270 (narrow), -N=C=O")));
     if (tmp_pt->element == "NCS")
-      AddPeak(2125.0, QString("NCS"), QString("~2125 (narrow), -N=C=S"));
+      AddPeak(2125.0, QString("NCS"), QString(tr("~2125 (narrow), -N=C=S")));
     if (tmp_pt->element == "OH")
-      AddPeak(3400.0, QString("OH"), QString("~3400 (broad), O-H"));
+      AddPeak(3400.0, QString("OH"), QString(tr("~3400 (broad), O-H")));
     if (tmp_pt->element == "HO")
-      AddPeak(3400.0, QString("OH"), QString("~3400 (broad), O-H"));
+      AddPeak(3400.0, QString("OH"), QString(tr("~3400 (broad), O-H")));
     if (tmp_pt->element == "NO2")
-      AddPeak(1525.0, QString("NO2"), QString("~1525 (narrow), -NO2"));
+      AddPeak(1525.0, QString("NO2"), QString(tr("~1525 (narrow), -NO2")));
     if (tmp_pt->element == "O2N")
-      AddPeak(1525.0, QString("NO2"), QString("~1525 (narrow), -NO2"));
+      AddPeak(1525.0, QString("NO2"), QString(tr("~1525 (narrow), -NO2")));
     if (tmp_pt->aromatic == true) {
-      AddPeak(1600.0, QString("aromatic"), QString("~1600 (narrow), aromatic ring C=C"));
-      AddPeak(1475.0, QString("aromatic"), QString("~1475 (narrow), aromatic ring C=C"));
+      AddPeak(1600.0, QString("aromatic"), QString(tr("~1600 (narrow), aromatic ring C=C")));
+      AddPeak(1475.0, QString("aromatic"), QString(tr("~1475 (narrow), aromatic ring C=C")));
     }
   }
 
@@ -361,8 +383,8 @@ void Molecule::CalcIR() {
 }
 
 // invoked from chemdata_tools.cpp
-// derived (well, butchered) from Lin S and Sandler SI, J. Phys. Chem. A ,
-// 200, 104, 7099-7105
+// derived (well, butchered) from Lin S-T and Sandler SI, J. Phys. Chem. A ,
+// 2000, 104(30), 7099-7105
 double Molecule::CalcKOW() {
   // get list of unique points
   up = AllPoints();
@@ -555,7 +577,7 @@ void Molecule::CalcpKa() {
 	  alt_pt1->hit = true;
 	  alt_pt2->hit = true;
 	  alt_pt3->hit = true;
-	  pKa_table.append( "10.5 (Guanidine)" );
+	  pKa_table.append( tr("10.5 (Guanidine)") );
 	}
       }
     }
@@ -575,19 +597,19 @@ void Molecule::CalcpKa() {
 	alt_pt2 = tmp_pt->neighbors.at(1);
 	if ( (alt_pt1->hosecode == "=OCC") &&
 	     (alt_pt1->hosecode == "=OCC") ) {
-	  pKa_table.append( "9 (a-carbon between two ketones)" );	  
+	  pKa_table.append( tr("9 (a-carbon between two ketones)") );	  
 	}
 	if ( (alt_pt1->hosecode == "=OCO") &&
 	     (alt_pt1->hosecode == "=OCC") ) {
-	  pKa_table.append( "11 (a-carbon between two ketones)" );	  
+	  pKa_table.append( tr("11 (a-carbon between two ketones)") );	  
 	}
 	if ( (alt_pt1->hosecode == "=OCC") &&
 	     (alt_pt1->hosecode == "=OCO") ) {
-	  pKa_table.append( "11 (a-carbon between ketone and ester)" );	  
+	  pKa_table.append( tr("11 (a-carbon between ketone and ester)") );	  
 	}
 	if ( (alt_pt1->hosecode == "=OCO") &&
 	     (alt_pt1->hosecode == "=OCO") ) {
-	  pKa_table.append( "13 (a-carbon between two esters)" );	  
+	  pKa_table.append( tr("13 (a-carbon between two esters)") );	  
 	}
       }
     }
@@ -596,16 +618,16 @@ void Molecule::CalcpKa() {
       if (tmp_pt->substituents == 1) {
 	alt_pt1 = tmp_pt->neighbors.at(0);  // MakeSSSR() should set this
 	if (alt_pt1->hosecode.contains("*") > 0) {
-	  pKa_table.append( "5 (Aromatic primary amine)" );
+	  pKa_table.append( tr("5 (Aromatic primary amine)") );
 	} else {
 	  if (alt_pt1->hosecode.contains("=") > 0) {
 	    if (alt_pt1->hosecode.contains("=O") > 0) {
 	      // amide!
 	    } else {
-	      pKa_table.append( "9-10 (Alkene-primary amine)" );
+	      pKa_table.append( tr("9-10 (Alkene-primary amine)") );
 	    }
 	  } else {
-	    pKa_table.append( "11 (Aliphatic primary amine)" );
+	    pKa_table.append( tr("11 (Aliphatic primary amine)") );
 	  }
 	}
       }
@@ -617,17 +639,17 @@ void Molecule::CalcpKa() {
 	if ( (alt_pt1->hosecode == "=CN(//)") &&
 	     (alt_pt2->hosecode == "=CN(//)") ) {
 	  // pyrrole
-	  pKa_table.append( "-4 (pyrrole)" );
+	  pKa_table.append( tr("-4 (pyrrole)") );
 	  continue;
 	}
 	if ( (alt_pt1->hosecode == "=CN(//)") &&
 	     (alt_pt2->hosecode == "=NN(//)") ) {
-	  pKa_table.append( "14.4 (secondary amine)" );
+	  pKa_table.append( tr("14.4 (secondary amine)") );
 	  continue;
 	}
 	if ( (alt_pt1->hosecode.contains("=O") == 0 ) &&
 	     (alt_pt2->hosecode.contains("=O") == 0 ) ) {
-	  pKa_table.append( "11 (aliphatic secondary amine)" );
+	  pKa_table.append( tr("11 (aliphatic secondary amine)") );
 	}
       }
       if (tmp_pt->substituents == 3) {  // check conjugated and tertiary amines
@@ -639,19 +661,19 @@ void Molecule::CalcpKa() {
 	  std::cout << "alt_pt2 = " << alt_pt2->hosecode << std::endl;
 	  if ( (alt_pt1->hosecode == "=CN(//)") &&
 	       (alt_pt2->hosecode == "=NN(//)") ) {
-	    pKa_table.append( "6.8 (conjugated secondary amine)" );
+	    pKa_table.append( tr("6.8 (conjugated secondary amine)") );
 	  }
 	  if ( (alt_pt1->hosecode.contains("*C*N(//)") == 1 ) &&
 	       (alt_pt2->hosecode.contains("*C*N(//)") == 1 ) ) {
-	    pKa_table.append( "5 (aromatic secondary amine)" );
+	    pKa_table.append( tr("5 (aromatic secondary amine)") );
 	  }
 	  if ( (alt_pt1->hosecode.contains("*N*N(//)") == 1 ) &&
 	       (alt_pt2->hosecode.contains("*C*N(//)") == 1 ) ) {
-	    pKa_table.append( "1-3 (purine/pyrimidine)" );
+	    pKa_table.append( tr("1-3 (purine/pyrimidine)") );
 	  }
 	  if ( (alt_pt1->hosecode.contains("=NC(//)") == 1 ) &&
 	       (alt_pt2->hosecode.contains("CN(//)") == 1 ) ) {
-	    pKa_table.append( "1-3 (purine/pyrimidine)" );
+	    pKa_table.append( tr("1-3 (purine/pyrimidine)") );
 	  }
 	}
 	if (tmp_pt->neighbors.count() == 3) {
@@ -662,13 +684,13 @@ void Molecule::CalcpKa() {
 	  if ( (alt_pt1->hosecode.contains("*") > 0 ) ||
 	       (alt_pt2->hosecode.contains("*") > 0 ) ||
 	       (alt_pt3->hosecode.contains("*") > 0 ) ) {
-	    pKa_table.append( "4 (tertiary amine)" );
+	    pKa_table.append( tr("4 (tertiary amine)") );
 	    continue;
 	  }
 	  if ( (alt_pt1->hosecode.contains("=O") == 0 ) &&
 	       (alt_pt2->hosecode.contains("=O") == 0 ) &&
 	       (alt_pt3->hosecode.contains("=O") == 0 ) ) {
-	    pKa_table.append( "10 (aliphatic tertiary amine)" );
+	    pKa_table.append( tr("10 (aliphatic tertiary amine)") );
 	  }
 	}
       }
@@ -678,7 +700,7 @@ void Molecule::CalcpKa() {
       if (tmp_pt->substituents == 1) {
 	alt_pt1 = tmp_pt->neighbors.at(0);  // MakeSSSR() should set this
 	if (alt_pt1->baseElement() == "O") {
-	  pKa_table.append( "8 (Peroxy acid)" );
+	  pKa_table.append( tr("8 (Peroxy acid)") );
 	}
 	if (alt_pt1->baseElement() == "S") {
 	  // sulfonic acid?
@@ -695,7 +717,7 @@ void Molecule::CalcpKa() {
 	      }
 	    }
 	    if ( (c_count == 1) && (o_count == 3) )
-	      pKa_table.append( "-1 (sulfonic acid)" );
+	      pKa_table.append( tr("-1 (sulfonic acid)") );
 	  }
 	}
 	if (alt_pt1->baseElement() == "C") {
@@ -709,24 +731,24 @@ void Molecule::CalcpKa() {
 	      } else {
 		//std::cout << "--" << alt_pt2->hosecode << std::endl;
 		if (alt_pt2->hosecode.contains("*") > 0) {
-		  pKa_table.append( "4 (Aromatic carboxylic acid)" );
+		  pKa_table.append( tr("4 (Aromatic carboxylic acid)") );
 		} else {
 		  int halide_count = alt_pt2->hosecode.contains("F") + alt_pt2->hosecode.contains("X") + alt_pt2->hosecode.contains("Y") + alt_pt2->hosecode.contains("I");
 		  if (halide_count == 0)
-		    pKa_table.append( "4.5 (Aliphatic carboxylic acid)" );
+		    pKa_table.append( tr("4.5 (Aliphatic carboxylic acid)") );
 		  if (halide_count == 1)
-		    pKa_table.append( "3 (Aliphatic carboxylic acid, a-halide)" );
+		    pKa_table.append( tr("3 (Aliphatic carboxylic acid, a-halide)") );
 		  if (halide_count == 2)
-		    pKa_table.append( "1.3 (Aliphatic carboxylic acid, 2 a-halide)" );
+		    pKa_table.append( tr("1.3 (Aliphatic carboxylic acid, 2 a-halide)") );
 		}
 	      }
 	    }
 	    continue;
 	  }
 	  if (alt_pt1->hosecode.contains("*") > 0) {
-	    pKa_table.append( "10 (Aromatic -OH)" );
+	    pKa_table.append( tr("10 (Aromatic -OH)") );
 	  } else {
-	    pKa_table.append( "16 (Aliphatic -OH)" );
+	    pKa_table.append( tr("16 (Aliphatic -OH)") );
 	  }
 	}
       }
@@ -736,9 +758,9 @@ void Molecule::CalcpKa() {
       if (tmp_pt->substituents == 1) {
 	alt_pt1 = tmp_pt->neighbors.at(0);  // MakeSSSR() should set this
 	if (alt_pt1->hosecode.contains("*") > 0) {
-	  pKa_table.append( "7.8 (Aromatic -SH)" );
+	  pKa_table.append( tr("7.8 (Aromatic -SH)") );
 	} else {
-	  pKa_table.append( "10.6 (Aliphatic -SH)" );
+	  pKa_table.append( tr("10.6 (Aliphatic -SH)") );
 	}
       }
     }
