@@ -161,9 +161,10 @@ double Molecule::CalculateRingAttachAngle(DPoint *t1) {
       tmp_pt = tmp_bond->otherPoint(t1);
       dx = dx + (tmp_pt->x - t1->x);
       dy = dy + (tmp_pt->y - t1->y);
+      break;
     }
   }
-
+  
   double ang = atan(dy / dx);
 
   if (dx > 0.0) ang = ang + 3.14159;
@@ -226,61 +227,11 @@ void Molecule::AllNeighbors() {
   }
 }
 
-// invoked from tool_2d3d.cpp...
-void Molecule::Make3DVersion() {
-  // get list of unique points
-  up = AllPoints();
-
-  cout << "Make3DVersion: points: " << up.count() << endl;
-
-  // find rings (esp. find aromaticity) - do after CopyTextToDPoint()
-  MakeSSSR();
-
-  // TODO: probably a force-field approach will work best here.
-  // Iterate thru until *minimum* changes occur
-  // Best approach seems to be to allow atoms to drift based on "pull"
-  // due to force-field (probably AMBER) paramters until minimum energy
-  // is found; obviously, bond length and angle are most significant,
-  // and probably the only parameters I will worry about
-
-  // probably a smart thing to do is to pre-place the most substituted atoms
-  QList<DPoint> mostSubstituted;
-  for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next() ) {
-    // copy coordinates.  This might help with chirality and double/triple...
-    tmp_pt->x3 = tmp_pt->x;  
-    tmp_pt->y3 = tmp_pt->y;
-    tmp_pt->z3 = tmp_pt->z;
-    if (tmp_pt->neighbors.count() == 4) mostSubstituted.append(tmp_pt);
-  }
-  if (mostSubstituted.count() == 0) {
-    for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next() ) {
-      if (tmp_pt->neighbors.count() == 3) mostSubstituted.append(tmp_pt);
-    }
-  }
-  if (mostSubstituted.count() == 0) {
-    for (tmp_pt = up.first(); tmp_pt != NULL; tmp_pt = up.next() ) {
-      if (tmp_pt->neighbors.count() == 2) mostSubstituted.append(tmp_pt);
-    }
-  }
-  if (mostSubstituted.count() == 0) {
-    QMessageBox::warning(0, tr("Unable to convert"), tr("Unable to convert the selected molecule to 3-D.") );
-    return;
-  }  
-  
-  // we're going to want the hydrogens added explicitly (maybe)
-  AddNMRprotons();
-
-  bool atMinimum = false;
-  double energyPrevLoop = 0.0, energyThisLoop = 0.0, de;
-  do {
-    de = fabs(energyPrevLoop - energyThisLoop);
-    if (de < 1.0)
-      atMinimum = true;
-  } while (atMinimum == false);
-}
-
 // invoke from chemdata_tools.cpp
-void Molecule::CalcPartialCharge() {
+// revised: now invoked from reactivity functions in Molecule
+// revised again:  moved to static function in MolData class
+double Molecule::CalcPartialCharge(QString atomtype) {
+  return 0.0;
 }
 
 Text *Molecule::CalcEmpiricalFormula(bool from_mw) {
@@ -508,7 +459,7 @@ Text *Molecule::CalcElementalAnalysis(bool show_dialog) {
   no = no * 100 / nmw;
   nn = nn * 100 / nmw;
 
-  ea.append("Elemental analysis:\n");
+  ea.append(tr("Elemental analysis:\n"));
   ea.append("C = ");
   n1.setNum(nc);
   ea.append(n1);
@@ -526,33 +477,33 @@ Text *Molecule::CalcElementalAnalysis(bool show_dialog) {
   ea.append(n1);
   ea.append("%");
 
+  /*
   int m1;
   if (show_dialog == true) {
-    m1 = QMessageBox::information(r, "Elemental analysis", ea, "Paste", "OK",
+    m1 = QMessageBox::information(r, tr("Elemental analysis"), ea, tr("Paste"), tr("OK"),
 				  QString::null, 1, 1);
   } else {
     m1 = 0;
   }
-  if (m1 == 0) {
-    cout << "paste" << endl;
-    double nx, ny;
+  */
 
-    QRect nr = BoundingBoxAll();
-    ny = nr.top() + 8.0;
-    nx = nr.right() + 16.0;
-
-    tmp_text = new Text(r);
-    //CHECK_PTR(tmp_text);
-    //cout << finalef << endl;
-    tmp_text->setJustify(JUSTIFY_TOPLEFT);
-    tmp_text->setText(ea);
-    ea.fill(' ');
-    tmp_text->setTextMask(ea);
-    tmp_pt = new DPoint(nx, ny);
-    tmp_text->setPoint(tmp_pt);
-    return tmp_text;
-  }
-  return 0;
+  double nx, ny;
+  
+  QRect nr = BoundingBoxAll();
+  ny = nr.top() + 8.0;
+  nx = nr.right() + 16.0;
+  
+  delete tmp_text;
+  tmp_text = new Text(r);
+  //CHECK_PTR(tmp_text);
+  tmp_pt = new DPoint(nx, ny);
+  tmp_text->setPoint(tmp_pt);
+  //std::cout << ea << std::endl;
+  tmp_text->setJustify(JUSTIFY_TOPLEFT);
+  tmp_text->setText(ea);
+  ea.fill(' ');
+  tmp_text->setTextMask(ea);
+  return tmp_text;
 }
 
 void Molecule::AddPeak(double d1, QString s1, QString s2) {
@@ -577,7 +528,7 @@ void Molecule::AddPeak(double d1, QString s1, QString s2) {
 // add hydrogens to atoms.  Only add to carbon if to_carbons == true
 void Molecule::AddHydrogens(bool to_carbon) {
   up = AllPoints();
-  int h, sumbonds;
+  int h = 0, sumbonds = 0;
   double dx;
   QString hnum, orig_element;
   int least_hindered_side;
@@ -677,3 +628,79 @@ void Molecule::AddHydrogens(bool to_carbon) {
   }
 }
 
+// reactivity:  use this for doing partial charges/VdW for now
+// int argument specifies analysis type
+void Molecule::Reactivity(int react_type) {
+  std::cout << "Reactivity(" << react_type << ")" << std::endl;
+  // get list of unique points
+  up = AllPoints();
+  // find aromatic rings
+  MakeSSSR();
+  // solve HOSE codes
+  Calc13CNMR(false);
+  // add hydrogens
+  cout << bonds.count() << endl;
+  AddNMRprotons();
+  up = AllPoints();  // need to re-run after adding protons
+  cout << bonds.count() << endl;
+
+  // reactivity... build list of all atom and bond retro-types
+  QString localatomtype;
+  double sumcharge = 0.0;
+  atomRxns.clear(); bondRxns.clear();
+  QListIterator<DPoint> atomrxniter(up);
+  for ( ; atomrxniter.current(); ++atomrxniter ) {
+    localatomtype = RetroAtomName( atomrxniter.current() );
+    atomRxns.append(localatomtype);
+    // set partial charges here
+    atomrxniter.current()->react_atom_type = localatomtype;
+    atomrxniter.current()->partial_charge = CalcPartialCharge(localatomtype);
+    sumcharge += MolData::partialCharge(localatomtype, 0);
+  }
+  std::cout << "Calc total charge: " << sumcharge << std::endl;
+  std::cout << "Atom count: " << atomRxns.count() << std::endl;
+  QListIterator<Bond> bondrxniter(bonds);
+  for ( ; bondrxniter.current(); ++bondrxniter ) {
+    if (bondrxniter.current()->isCHBond() == false) {
+      bondRxns.append(RetroBondName( bondrxniter.current() ));
+      bondrxniter.current()->setCName(RetroBondName( bondrxniter.current() ) );
+      //bondrxniter.current()->setReactions(BuildReactionList(bondrxniter.current()->CName()));
+    }
+  }
+  std::cout << "Bond count: " << bondRxns.count() << std::endl;
+
+  // do stuff, based on value passed in arg
+  if (react_type == MODE_TOOL_REACTIVITY_FORWARD) {
+    // for reactivity, we should look at atom types first, since atoms will
+    // be the likely target of forward reactions.
+    std::cout << "Atom type list for FORWARD:" << std::endl;
+    for ( QStringList::Iterator it = atomRxns.begin(); 
+	  it != atomRxns.end(); ++it ) {
+        std::cout << *it << std::endl;
+    }
+    std::cout << "--- list ends ---" << std::endl;
+    // we will also have to consider other atom types and bonds that may
+    // interfere with a given reaction
+  }
+  if (react_type == MODE_TOOL_REACTIVITY_RETRO) {
+    // for retrosynthesis, we are more interested in bonds, to se how they
+    // might have been formed
+    std::cout << "Bond type list for RETRO:" << std::endl;
+    for ( QStringList::Iterator it = bondRxns.begin(); 
+	  it != bondRxns.end(); ++it ) {
+      std::cout << *it << std::endl;
+    }
+    std::cout << "--- list ends ---" << std::endl;
+    // we will also have to consider R group substitution here.
+    // call Molecule::Retro() (in retro.cpp) to do matching
+    int nr = Retro();
+  }
+  if (react_type == MODE_TOOL_CHARGES) {
+    // just look at atom types and assign AMBER-like partial charges
+    std::cout << "Sum of charges = " << sumcharge << std::endl;
+  }
+
+  // remove added hydrogens
+  RemoveNMRprotons();
+  cout << bonds.count() << endl;
+}
