@@ -16,6 +16,7 @@ public:
   MolInfoDialog(QWidget *parent, QString name);
   bool isMWChecked() { return mw_check->isChecked(); }
   bool isEFChecked() { return ef_check->isChecked(); }
+  bool isEAChecked() { return ea_check->isChecked(); }
   void setMW(QString s1) { 
     mw_check->setText(s1); 
   }
@@ -25,7 +26,8 @@ public:
     ef_check->setText(ef); 
   }
   void setEA(QString s1) { 
-    //ea_label->setText(s1); 
+    ea = s1;
+    ea_check->setText(ea); 
   }
   void setCAS(QString s1) { 
     if (s1 != "unknown") {
@@ -41,10 +43,14 @@ public:
 
   public slots:
     void SendHelp();
+  void doAccept();
+
+ signals:
+  void MIDClose();
 
  private:
-  QCheckBox *mw_check, *ef_check, *dg_check;
-  QLabel *ea_label, *name_label, *cas_label;
+  QCheckBox *mw_check, *ef_check, *dg_check, *ea_check;
+  QLabel *name_label, *cas_label;
   QString mw, ea, ef, dg, casno;
 };
 
