@@ -4,6 +4,7 @@
 #include <qobject.h>
 #include <qnetworkprotocol.h>
 #include <qurloperator.h>
+#include <qhttp.h>
 
 class QString;
 class QStringList;
@@ -17,14 +18,18 @@ public:
   QString getCanonicalSmiles(QString, QString);
   QString Rearrange(QString);
   bool getNameCAS(QString, QString);
+  bool get3DMol(QString);
+  bool runBuild3D(QString);
   bool status;
-  QString htfile, sname, scas;
-  QUrlOperator op;
+  QString htfile, sname, scas, s3dmol;
+  QUrlOperator *op;
+  QHttp hp1;
+  QNetworkOperation *op1;
 
   public slots:
     void slotData(const QByteArray &);
   void slotFinished(QNetworkOperation *);
-
+  void rf(bool);
 };
 
 #endif
