@@ -46,14 +46,17 @@ Render2D::Render2D(QWidget *parent, const char *name)
   prevpos = 0;
   mouse1down = false;
   mouse3down = false;
+  doubleClickFlag = false;
   dragging = false;
   moved = false;
 
   highlightpoint = 0;
   highlightobject = 0;
   localtext = 0;
+  localtexteditor = 0;
 
   thick = 1;
+  savedBondOrder = -1;
   zoomFactor = 1.0;
 
   currentFont = QFont("Helvetica", 12);
@@ -82,6 +85,7 @@ void Render2D::ReactionAnalysis(int x) {
 // this function required for good behavior if user changes tools while
 // text is selected.
 void Render2D::CleanUpAfterTextTool() {
+  /*  OLD CODE
   if (localtext == 0) return;  // don't need to clean up if no text.
   // save text, clean up
   localtext->DeselectAllText();
@@ -99,6 +103,11 @@ void Render2D::CleanUpAfterTextTool() {
     highlightobject->Highlight(false);
     highlightobject = 0;
   }
+  */
+
+  if (localtexteditor != 0) {
+    DrawText_FinishText();
+  }
 }
 
 void Render2D::setClipboard(Clipboard *clip1) { c->setClipboard(clip1); }
@@ -194,7 +203,8 @@ void Render2D::setMode_DrawWavyLine() {
   repaint(false);
 }
 
-void Render2D::setMode_DrawArrow(QString btype) { 
+void Render2D::setMode_DrawArrow(QString btype) {
+  bracket_type = 0;
   if (mode == MODE_TEXT) CleanUpAfterTextTool();
   if (mode == MODE_RING) smartplace = 0;
   mode = MODE_DRAWARROW; 
@@ -202,6 +212,8 @@ void Render2D::setMode_DrawArrow(QString btype) {
   setCursor(crossCursor);
   c->DeselectAll();
   if (btype == "REGULAR") bracket_type = ARROW_REGULAR;
+  if (btype == "TOP_HARPOON") bracket_type = ARROW_TOPHARPOON;
+  if (btype == "BOTTOM_HARPOON") bracket_type = ARROW_BOTTOMHARPOON;
   if (btype == "DIDNT_WORK") bracket_type = ARROW_DIDNT_WORK;
   if (btype == "DASH") bracket_type = ARROW_DASH;
   if (btype == "BI1") bracket_type = ARROW_BI1;
@@ -398,6 +410,9 @@ void Render2D::Tool(int t) {
   case MODE_TOOL_TEST:
     emit SignalSetStatusBar( tr("Tool test mode") );
     break;
+  default:
+    emit SignalSetStatusBar( tr("Click on a molecule") );
+    break;
   }
 }
 
@@ -443,6 +458,15 @@ void Render2D::Copy() {
 }
 
 void Render2D::Paste() {
+  if (mode == MODE_TEXT) {
+    if (localtext != 0) { 
+      QClipboard* clip=QApplication::clipboard();
+      QString pastext = clip->text(QClipboard::Clipboard);
+      localtext->InsertString(pastext);
+      repaint(false); 
+    }
+    return;  // eliminate unpredictable behavior
+  }
   c->StartUndo(0, 0);
   if (c->Paste() == false) return;
   c->Move(30,30);
@@ -483,6 +507,9 @@ void Render2D::EraseSelected() {
   }
   c->StartUndo(0, 0);
   c->EraseSelected();
+  // in all likelihood, we just blew away highlightobject.
+  // if not (unlikely!), user can reselect it.
+  highlightobject = 0;
   if (mode == MODE_SELECT_MULTIPLE_SELECTED) mode = MODE_SELECT;
   repaint(false);
 }
@@ -542,6 +569,8 @@ void Render2D::Inserted() {
 void Render2D::SetFont(QFont f) {
   if (localtext != 0) {
     localtext->setFont(f);
+    if (localtexteditor != 0)
+      localtexteditor->setCurrentFont(f);
     repaint(false);
     return;
   } else {
@@ -554,6 +583,8 @@ void Render2D::SetColor(QColor c1) {
   if (mode != MODE_SELECT_MULTIPLE_SELECTED) {
     currentColor = c1;
     if (localtext != 0) localtext->SetColor(c1);
+    if (localtexteditor != 0)
+      localtexteditor->setColor(c1);
   } else {
     c->SetColorIfHighlighted(c1);
   }
