@@ -98,8 +98,8 @@ void Render2D::drawLine(QPoint a, QPoint b, int t, QColor c1, int s) {
   if (s == 0) // solid
     //p.setPen(QPen(c1, t - 1));
     p.setPen(QPen(c1, t));
-  else // dashed
-    p.setPen(QPen(c1,1,DotLine));
+  else // dashed -- always apply thickness
+    p.setPen(QPen(c1, t, DotLine));
   p.drawLine(a, b);
 }
 
@@ -134,7 +134,7 @@ void Render2D::drawBezier(QPointArray a1, QColor c1, bool drawPoints,
     ar1 = a1.point( a1.count() - 2 );
     ar2 = a1.point( a1.count() - 1 );
 
-    double ang, oppang, myangle, x2, y2;
+    double ang;
     ang = getAngle(ar2, ar1);
 
     double newang1 = ang + 30.0;
@@ -255,7 +255,7 @@ void Render2D::drawWavyLine(QPoint a, QPoint b, QColor c1) {
   double dmag = sqrt(dx*dx + dy*dy);
   int nwaves = my_round(dmag / 6.0);
   double wwidth = dmag / (double)nwaves;
-  int cn, lx, ly;
+  int cn;
   double dx1, dy1, ia_rad, dx2, dy2;
   for (cn = 0; cn <= (int)dmag; cn++) {
     dx1 = (double)cn;
@@ -375,7 +375,7 @@ void Render2D::drawDownLine(QPoint a, QPoint b, QColor c1) {
   drawLine(t1, t2, 1, c1);
 }
 
-void Render2D::drawArrow(QPoint a, QPoint b, QColor c1, int s1) {
+void Render2D::drawArrow(QPoint a, QPoint b, QColor c1, int s1, int th1) {
   // Build arrowhead(s)
   double ang, oppang, myangle, x2, y2;
   oppang = getAngle(a, b);
@@ -398,9 +398,23 @@ void Render2D::drawArrow(QPoint a, QPoint b, QColor c1, int s1) {
 	     my_round(a.y() + (sin(newang4/MOL_ARAD) * 10.0)) );
 
   if (s1 == ARROW_REGULAR) {
-    drawLine(a, b, 1, c1);
-    drawLine(b, a1, 1, c1);
-    drawLine(b, a2, 1, c1);
+    drawLine(a, b, th1, c1);
+    drawLine(b, a1, th1, c1);
+    drawLine(b, a2, th1, c1);
+  }
+  if (s1 == ARROW_TOPHARPOON) {
+    drawLine(a, b, th1, c1);
+    if (a1.y() < a2.y())
+      drawLine(b, a1, th1, c1);
+    else
+      drawLine(b, a2, th1, c1);
+  }
+  if (s1 == ARROW_BOTTOMHARPOON) {
+    drawLine(a, b, th1, c1);
+    if (a1.y() > a2.y())
+      drawLine(b, a1, th1, c1);
+    else
+      drawLine(b, a2, th1, c1);
   }
   if (s1 == ARROW_DIDNT_WORK) {
     QPoint m1 = Midpoint(a,b);
@@ -416,18 +430,18 @@ void Render2D::drawArrow(QPoint a, QPoint b, QColor c1, int s1) {
 	       my_round(m1.y() + (sin(newang7/MOL_ARAD) * 10.0)) );
     QPoint a8( my_round(m1.x() + (cos(newang8/MOL_ARAD) * 10.0)),
 	       my_round(m1.y() + (sin(newang8/MOL_ARAD) * 10.0)) );
-    drawLine(a, b, 1, c1);
-    drawLine(b, a1, 1, c1);
-    drawLine(b, a2, 1, c1);
-    drawLine(m1, a5, 1, c1);
-    drawLine(m1, a6, 1, c1);
-    drawLine(m1, a7, 1, c1);
-    drawLine(m1, a8, 1, c1);
+    drawLine(a, b, th1, c1);
+    drawLine(b, a1, th1, c1);
+    drawLine(b, a2, th1, c1);
+    drawLine(m1, a5, th1, c1);
+    drawLine(m1, a6, th1, c1);
+    drawLine(m1, a7, th1, c1);
+    drawLine(m1, a8, th1, c1);
   }
   if (s1 == ARROW_DASH) {
-    drawLine(a, b, 1, c1, 1);
-    drawLine(b, a1, 1, c1);
-    drawLine(b, a2, 1, c1);
+    drawLine(a, b, th1, c1, 1);
+    drawLine(b, a1, th1, c1);
+    drawLine(b, a2, th1, c1);
   }
   if (s1 == ARROW_BI1) {
     drawLine(a, b, 1, c1);
@@ -573,6 +587,7 @@ void Render2D::drawCircle(QPoint a, int rad, QColor cfill) {
 }
 
 void Render2D::drawText(QChar ch, QPoint a, QColor c1, QFont f) {
+  std::cout << "drawText: " << ch << std::endl;
   int fsize, dy;
   fsize = f.pointSize();
   if (fsize < 1) fsize = f.pixelSize();
