@@ -20,87 +20,102 @@ using std::endl;
 #include "text.h"
 #include "bracket.h"
 #include "defs.h"
+#include "charsel.h"
+
+// insert symbol, from Application
+void Render2D::InsertSymbol() {
+  if (localtexteditor == 0) return;
+
+  CharSelDialog cd1(this, "charsel");
+  QFont tf1;
+  if (cd1.exec()) {
+    tf1 = localtexteditor->currentFont();
+    localtexteditor->setCurrentFont( cd1.selectedFont() );
+    localtexteditor->insert( cd1.selectedChar() );
+    localtexteditor->setCurrentFont( tf1 );
+  }
+}
+
+// events from QTextEdit
+void Render2D::DrawText_returnPressed(void) {
+  std::cout << "Return pressed inside QTextEdit" << std::endl;
+  if ( localtext->Justify() == JUSTIFY_CENTER ) {
+    // this appears to cause a seg fault only under this condition,
+    // called from signal...
+    DrawText_FinishText();
+    return;
+  }
+}
+
+void Render2D::DrawText_textChanged(void) {
+  std::cout << "contents: " << localtexteditor->contentsWidth() << ",";
+  std::cout << localtexteditor->contentsHeight() << std::endl;
+  localtexteditor->resize( localtexteditor->contentsWidth() + 21,
+			   localtexteditor->contentsHeight() );
+}
+
+void Render2D::DrawText_CVAchanged(QTextEdit::VerticalAlignment cva) {
+  std::cout << "New vertical alignment: " << (int)cva << std::endl;
+}
+
+void Render2D::DrawText_FinishText() {
+  emit TextOff();
+  QString lt2 = localtexteditor->text();
+  localtexteditor->setTextFormat(Qt::PlainText);
+  QString lt1 = localtexteditor->text();
+  if (localtext->Justify() == JUSTIFY_CENTER) {
+    lt1.remove('\n');
+  }
+  localtext->setText( lt1, lt2 );
+  lt1.fill(' ');
+  localtext->setTextMask( lt1 );
+  if (localtext->getText().length() > 0) { 
+    if (!text_exists) c->addText(localtext); 
+  } else {
+    localtext->Start()->element = "C";
+    c->Erase(localtext);
+  }
+  if (localtext->Justify() == JUSTIFY_TOPLEFT) {
+    delete localtexteditor;
+    localtexteditor = 0;
+  } else {
+    localtexteditor->hide();
+    localstring = "XDC_DELETE_ME";
+  }
+  localtext = 0;
+  highlightpoint = 0;
+  if (highlightobject != 0) {
+    highlightobject->Highlight(false);
+  }      
+  highlightobject = 0;
+  if (doubleClickFlag) {
+    mode = prev_mode;
+    doubleClickFlag = false;
+  }
+  return;
+}
 
 // mouse events to handle text functions
 // (button states should be set before we get here)
 void Render2D::DrawText_mousePressEvent(QMouseEvent *e1, QPoint cqp) {
-  text_drag = false;
-  if (localtext != 0) {
-    DPoint *e = new DPoint;
-    e->x = e1->x(); e->y = e1->y();
-    if (localtext->WithinBounds(e) == true) {
-      // move cursor, select, or something...
-      text_drag = true;
-      start_drag = e;
-    }
-  }
-  // do nothing, wait for release
-}
 
-void Render2D::DrawText_mouseReleaseEvent(QMouseEvent *e1, QPoint cqp) {
-  if (localtext != 0) {  // if already working on something, change behavior
-    DPoint *e = new DPoint;
-    e->x = e1->x(); e->y = e1->y();
-    if (text_drag == true) {
-      double dlta = fabs((start_drag->x - e->x) + (start_drag->y - e->y));
-      if (dlta < 0.5) { 
-	localtext->MoveCursor(e); 
-	text_drag = false;
-	repaint(false); 
-	return; 
-      }
-      // finish multiple selection
-      localtext->Select(start_drag, e);
-      repaint(false);
-      text_drag = false;
-      return;
-    }
-    if (localtext->WithinBounds(e) == true) {
-      // move cursor, select, or something...
-    } else {
-      // save text, clean up
-      localtext->DeselectAllText();
-      emit TextOff();
-      if (localtext->getMolecule() != 0)
-	localtext->getMolecule()->Changed();
-      if (!text_exists) {
-	if (localtext->getText().length() > 0)
-	  { c->addText(localtext); localtext = 0; }
-      } else {
-	if (localtext->getText().length() == 0) { 
-	  // reset underlying point to "C" carbon
-	  localtext->Start()->element = "C";
-	  localtext->Start()->elementmask = " ";
-	  c->Erase(localtext); localtext = 0; 
-	}
-	if (localtext->getText() == "") { 
-	  // reset underlying point to "C" carbon
-	  localtext->Start()->element = "C";
-	  localtext->Start()->elementmask = " ";
-	  c->Erase(localtext); localtext = 0; 
-	}
-      }
-      localtext = 0;
-      highlightpoint = 0;
-      if (highlightobject != 0) {
-	highlightobject->Highlight(false);
-	highlightobject = 0;
-      }
-      repaint(false);
-    }
-
-    double distobj;
-    Drawable *no = c->FindNearestObject(e, distobj);
-    if ( (no != 0) && (no->Type() == TYPE_TEXT) ) { // jump to closest text
-      highlightpoint = 0;
-      highlightobject = no;
-    } else {
-      return;
-    }
+  // experimental: use QTextEdit to edit text!
+  if (localtexteditor != 0) {
+    // if clicked off localtexteditor...
+    std::cout << "click outside editor" << std::endl;
+    DrawText_FinishText();
+    return;
   }
 
+  if (localtexteditor == 0) {
+    localtexteditor = new QTextEdit(this, "local text editor");
+    localtexteditor->setTextFormat(Qt::RichText);
+    super_set = false; sub_set = false;
+  }
   if (highlightobject != 0) { // edit existing object
     if (highlightobject->Type() == TYPE_BRACKET) {
+      delete localtexteditor;
+      localtexteditor = 0;
       Bracket *this_bracket = (Bracket *)highlightobject;
       bool ok = false;
       QString btext = QInputDialog::getText(tr( "Enter subscript" ),
@@ -115,34 +130,52 @@ void Render2D::DrawText_mouseReleaseEvent(QMouseEvent *e1, QPoint cqp) {
 
     text_exists = true;
     localtext = (Text *)highlightobject;
-    if (localtext->getText() == "C") {
-      localtext->setText("");
-      localtext->setTextMask("");
-    }
-    emit TextOn(localtext->getFont());
-    repaint(false);
-  } else {  // create new object
+    Q_CHECK_PTR(localtext);
+  }
+  localtexteditor->setText( "" );
+  if (localtext != 0) {
+    localtexteditor->setCurrentFont( localtext->getFont() );
+    localtexteditor->setColor( localtext->GetColor() );
+    localtexteditor->setText( localtext->getRichText() );
+    localtexteditor->move( localtext->Start()->toQPoint() );
+    //localtexteditor->resize( localtext->BoundingBox().width() + 40,
+    //		     localtext->BoundingBox().height() + 20 );
+    localtexteditor->resize( localtexteditor->contentsWidth() + 21,
+			     localtexteditor->contentsHeight() );
+  } else {
     text_exists = false;
+    localtexteditor->move( e1->pos() );
     localtext = new Text(this);
     localtext->setFont(currentFont);
     localtext->SetColor(currentColor);
-    DPoint *e = new DPoint;
-    //e->x = e1->x(); e->y = e1->y();
-    e->set(cqp);
-    emit TextOn(localtext->getFont());
     if (highlightpoint) { 
       localtext->setPoint(highlightpoint);
       localtext->setJustify(JUSTIFY_CENTER);
       localtext->setText("");
       localtext->setTextMask("");
     } else {
-      localtext->setPoint(e);
+      localtext->setPoint( new DPoint(cqp) );
       localtext->setJustify(JUSTIFY_TOPLEFT);
       localtext->setText("");
       localtext->setTextMask("");
     }
-    repaint(false);
+    localtexteditor->setCurrentFont( localtext->getFont() );
+    localtexteditor->setColor( localtext->GetColor() );
   }
+  emit TextOn( localtext->getFont() );
+  connect( localtexteditor, SIGNAL( returnPressed() ), this,
+	   SLOT( DrawText_returnPressed() ) );
+  connect( localtexteditor, SIGNAL( textChanged() ), this,
+	   SLOT( DrawText_textChanged() ) );
+  localtexteditor->show();
+  localtexteditor->setFocus();
+  return;
+}
+
+void Render2D::DrawText_mouseReleaseEvent(QMouseEvent *e1, QPoint cqp) {
+  // short circuit for now!
+  return;
+  // end QTextEdit experiment
 }
 
 void Render2D::DrawText_mouseMoveEvent(QMouseEvent *e1) {
@@ -163,8 +196,9 @@ void Render2D::DrawText_mouseMoveEvent(QMouseEvent *e1) {
     if (text_drag == true) {
       localtext->Select(start_drag, e);
       repaint(false);
+      return;
     }
-    return;
+    //return;
   }
 
   // look for place to draw if no current text object
@@ -216,73 +250,18 @@ void Render2D::keyPressEvent(QKeyEvent *k) {
     }
   }
 
-  // discard delete key (to avoid random behavior)
-  if (k->key() == Key_Delete) {
-    std::cerr << "EXCEPTION: Delete key caught." << endl;
-    return;
-  }
-
-  // if text object is active, do something.
-  if (localtext != 0) {
-    // if escape character pressed, save object
+  // new stuff for localtexteditor
+  if (localtexteditor != 0) {
     if (k->key() == Key_Escape) {
-      emit TextOff();
-      if (localtext->getMolecule() != 0)
-	localtext->getMolecule()->Changed();
-      if (!text_exists) {
-	if (localtext->getText().length() > 0)
-	  c->addText(localtext);
-      } else {
-	if (localtext->getText().length() == 0) {
-	  // reset underlying point to "C" carbon
-	  localtext->Start()->element = "C";
-	  c->Erase(localtext);
-	}
-      }
-      localtext = 0;
-      highlightpoint = 0;
-      if (highlightobject != 0) {
-	highlightobject->Highlight(false);
-	highlightobject = 0;
-      }
-      repaint(false);
+      DrawText_FinishText();
       return;
     }
-    // if return pressed, and Justify == JUSTIFY_CENTER...
-    if ( (k->key() == Key_Return) && 
-	 (localtext->Justify() == JUSTIFY_CENTER) ) {
-      emit TextOff();
-      if (localtext->getMolecule() != 0)
-	localtext->getMolecule()->Changed();
-      if (!text_exists) {
-	if (localtext->getText().length() > 0)
-	  c->addText(localtext);
-      } else {
-	if (localtext->getText().length() == 0) {
-	  // reset underlying point to "C" carbon
-	  localtext->Start()->element = "C";
-	  c->Erase(localtext);
-	}
-      }
-      localtext = 0;
-      highlightpoint = 0;
-      if (highlightobject != 0) {
-	highlightobject->Highlight(false);
-      }      
-      highlightobject = 0;  // it was deleted by ChemData::Erase().
-      // ChemData etc. do this for us now.
-      //if (highlightobject != 0) {
-      //	highlightobject->Highlight(false);
-      //	highlightobject = 0;
-      //}
-      repaint(false);
+    if (k->key() == Key_Delete) {
+      std::cerr << "EXCEPTION: Delete key caught." << endl;
       return;
     }
-
-    // if not escape, just insert.
-    localtext->InsertCharacter(k);
-    repaint(false);
   }
+  return;
 }
 
 // Superscript selected text
@@ -291,6 +270,13 @@ void Render2D::Superscript() {
     localtext->DoSuperscript();
     repaint(false);
   }
+  if (super_set) {
+    localtexteditor->setVerticalAlignment(QTextEdit::AlignNormal);
+    super_set = false;
+  } else {
+    localtexteditor->setVerticalAlignment(QTextEdit::AlignSuperScript);
+    super_set = true;
+  }
 }
 
 // Subscript selected text
@@ -299,6 +285,13 @@ void Render2D::Subscript() {
     localtext->DoSubscript();
     repaint(false);
   }
+  if (sub_set) {
+    localtexteditor->setVerticalAlignment(QTextEdit::AlignNormal);
+    sub_set = false;
+  } else {
+    localtexteditor->setVerticalAlignment(QTextEdit::AlignSubScript);
+    sub_set = true;
+  }
 }
 
 // Left-justify text
@@ -307,6 +300,9 @@ void Render2D::JustifyLeft() {
     localtext->ForceAlignment(TEXT_LALIGN);
     repaint(false);
   }
+  localtexteditor->selectAll();
+  localtexteditor->setAlignment(Qt::AlignLeft);
+  localtexteditor->selectAll(false);
 }
 
 // Center text
@@ -315,6 +311,9 @@ void Render2D::JustifyCenter() {
     localtext->ForceAlignment(TEXT_CALIGN);
     repaint(false);
   }
+  localtexteditor->selectAll();
+  localtexteditor->setAlignment(Qt::AlignCenter);
+  localtexteditor->selectAll(false);
 }
 
 // Right-justify text
@@ -323,6 +322,9 @@ void Render2D::JustifyRight() {
     localtext->ForceAlignment(TEXT_RALIGN);
     repaint(false);
   }
+  localtexteditor->selectAll();
+  localtexteditor->setAlignment(Qt::AlignRight);
+  localtexteditor->selectAll(false);
 }
 
 // Bold selected text
@@ -331,6 +333,9 @@ void Render2D::Bold() {
     localtext->DoBold();
     repaint(false);
   }
+  if (localtexteditor != 0) {
+    localtexteditor->setBold( !localtexteditor->bold() );
+  }
 }
 
 // Italicize selected text
@@ -339,6 +344,9 @@ void Render2D::Italic() {
     localtext->DoItalic();
     repaint(false);
   }
+  if (localtexteditor != 0) {
+    localtexteditor->setItalic( !localtexteditor->italic() );
+  }
 }
 
 // Underline selected text
@@ -347,6 +355,9 @@ void Render2D::Underline() {
     localtext->DoUnderline();
     repaint(false);
   }
+  if (localtexteditor != 0) {
+    localtexteditor->setUnderline( !localtexteditor->underline() );
+  }
 }
 
 // origin = origin point of text
