@@ -66,10 +66,11 @@ QString Text::ToXML(QString xml_id) {
   s.append("</textstring>\n");
 
   // write textmask
-  s.append("<textmask>");
-  s.append(start->elementmask);
-  s.append("</textmask>\n");
+  s.append("<richstring>");
+  s.append(displayText);
+  s.append("</richstring>\n");
 
+  /* QTextEdit saves these attributes as HTML in the <richstring> now
   // write color
   s.append("<color>");
   n1.setNum(color.red());
@@ -89,8 +90,9 @@ QString Text::ToXML(QString xml_id) {
   n1.setNum(fsize);
   s.append(n1);
   s.append("</font>\n");
+  */
 
-  // end arrow
+  // end text
   s.append("</text>\n");
 
   return s;
@@ -134,6 +136,9 @@ void Text::FromXML(QString xml_tag) {
   i1 = xml_tag.find("<textstring>");
   i2 = xml_tag.find("</textstring>") + 13;
   SetTextstringFromXML(xml_tag.mid(i1, i2 - i1));
+  i1 = xml_tag.find("<richstring>");
+  i2 = xml_tag.find("</richstring>") + 13;
+  SetRichstringFromXML(xml_tag.mid(i1, i2 - i1));
   i1 = xml_tag.find("<textmask>");
   i2 = xml_tag.find("</textmask>") + 11;
   SetTextmaskFromXML(xml_tag.mid(i1, i2 - i1));
@@ -168,9 +173,30 @@ void Text::SetTextstringFromXML(QString xml_tag) {
     start->element = xml_tag;
 }
 
+// get Text::displayText from <richstring> tag
+void Text::SetRichstringFromXML(QString xml_tag) {
+  cout << "SetRichstringFromXML:" << xml_tag << endl;
+  int i1, i2;
+  //int d1, d2, d3;
+
+  i1 = xml_tag.find("<richstring>");
+  i2 = xml_tag.find("</richstring>");
+  xml_tag.remove(i2, 999);
+  xml_tag.remove(i1, 12);
+  if (start != 0)
+    displayText = xml_tag;
+}
+
 // get Text::textmask from <textmask> tag
 void Text::SetTextmaskFromXML(QString xml_tag) {
-  cout << "SetTextmaskFromXML:" << xml_tag << endl;
+  //cout << "SetTextmaskFromXML:" << xml_tag << endl;
+
+  // obsolete tag!  silently update...
+  // note that user may need to reformat text
+  displayText = start->element;
+  displayText.replace("\n", "</p>");
+
+  /*
   int i1, i2;
   //int d1, d2, d3;
 
@@ -180,6 +206,7 @@ void Text::SetTextmaskFromXML(QString xml_tag) {
   xml_tag.remove(i1, 10);
   if (start != 0)
     start->elementmask = xml_tag;
+  */
 }
 
 bool Text::WithinRect(QRect n, bool shiftdown) {
@@ -190,31 +217,149 @@ bool Text::WithinRect(QRect n, bool shiftdown) {
   return highlighted;
 }
 
-void Text::Render(void)
-{
-  UpdateDisplayText();
-  int supersub = r->GetTextHeight(font) / 2;
-  QFont supersubfont = subscriptFont(font);
-  int subx = r->GetTextHeight(supersubfont) / 2;
+void Text::Render(void) {
+  //std::cout << "displayText.length():" << displayText.length() << std::endl;
+
+  bool boldFlag = false, italicFlag = false, underlineFlag = false,
+    superFlag = false, subFlag = false;
+  QFont currentFont = font;
+  QColor currentColor = color;
+
+  QStringList parseList, renderList, lineWidths;
+
+  int lineheight = 0, linewidth = 0, textheight = 0, textwidth = 0;;
+  int cp1, lp1, lp2;
+  QString n1, currentElement = "";
+  bool flag1 = false;
+  for (cp1 = 0; cp1 < displayText.length() ; cp1++ ) {
+    flag1 = true;
+    if (displayText[cp1] == '<') {
+      parseList.append(currentElement);
+      currentElement = "<";
+      flag1 = false;      
+    }
+    if (displayText[cp1] == '>') {
+      currentElement.append(displayText[cp1]);
+      parseList.append(currentElement);
+      currentElement = "";
+      flag1 = false;
+    }
+    if (flag1) {
+      currentElement.append(displayText[cp1]);
+    }
+  }
+
+  // just in case there were no tags...
+  if (currentElement.length() > 0) parseList.append(currentElement);
+
+  // mock render to determine width and height
+  QRect b1;
+  for ( QStringList::Iterator it = parseList.begin(); it != parseList.end(); 
+	++it ) {
+    currentElement = *it;
+    if (currentElement == "") continue;
+    if (currentElement == " ") continue;
+    if (currentElement == "\n") continue;
+    if (currentElement == "<p>") continue;
+    if (currentElement == "<html>") continue;
+    if (currentElement == "</html>") continue;
+    if (currentElement == "<head>") continue;
+    if (currentElement == "</head>") continue;
+    if (currentElement == "</body>") continue;
+    if (currentElement.left(3) == "<p ") continue;
+    if (currentElement.left(6) == "<body ") continue;
+    if (currentElement.left(6) == "<meta ") continue;
+    if (currentElement == "</span>") continue;
+    currentElement.replace("&lt;", "<");
+    currentElement.replace("&gt;", ">");
+    renderList.append(currentElement);
+    if (currentElement == "</p>") {
+      n1.setNum(linewidth);
+      lineWidths.append(n1);
+      if (linewidth > textwidth) textwidth = linewidth;
+      textheight += lineheight;
+      linewidth = 0;
+      lineheight = 0;
+      continue;
+    }
+    if (currentElement.left(6) == "<span ") {
+      // set font
+      //std::cout << "span: " << currentElement << std::endl;
+      boldFlag = false, italicFlag = false, underlineFlag = false,
+	superFlag = false, subFlag = false;      
+      if (currentElement.contains("font-family") > 0) {
+	lp2 = currentElement.find("font-family");
+	lp1 = currentElement.find(":", lp2);
+	lp2 = currentElement.find(";", lp1);
+	currentFont.setFamily( currentElement.mid(lp1 + 1, lp2 - lp1 - 1) );
+      }
+      if (currentElement.contains("font-size") > 0) {
+	lp2 = currentElement.find("font-size");
+	lp1 = currentElement.find(":", lp2);
+	lp2 = currentElement.find("pt", lp1);
+	currentFont.setPointSize( currentElement.mid(lp1 + 1, lp2 - lp1 - 1).toInt() );
+      }
+      currentFont.setBold(false);
+      currentFont.setItalic(false);
+      currentFont.setUnderline(false);
+      if (currentElement.contains("font-weight") > 0) {
+	boldFlag = true;
+	currentFont.setBold(true);
+      }
+      if (currentElement.contains("italic") > 0) {
+	italicFlag = true;
+	currentFont.setItalic(true);
+      }
+      if (currentElement.contains("underline") > 0) {
+	underlineFlag = true;
+	currentFont.setUnderline(true);
+      }
+      if (currentElement.contains("super") > 0) {
+	superFlag = true;
+      }
+      if (currentElement.contains("sub") > 0) {
+	subFlag = true;
+      }
+      continue;
+    }
+    b1 = r->GetTextDimensions(currentElement, currentFont);
+    if (b1.height() > lineheight) lineheight = b1.height();
+    linewidth += b1.width();
+  }
+
+  if (lineheight > textheight) textheight = lineheight;
+  if (linewidth > textwidth) textwidth = linewidth;
+
+  /*
+  std::cout << displayText << std::endl;
+  std::cout << "Text dimensions = " << textwidth << "," << textheight 
+	    << std::endl;  
+  std::cout << "---" << std::endl;
+  */
+
+  int supersub, subx;
+  QFont supersubfont;
   QColor drawcolor;
   if (highlighted)
     drawcolor = QColor(255,0,0);
   else
     drawcolor = color;
-  QRect b = r->GetTextDimensions(displayText, font);
+  QRect b(0, 0, textwidth, textheight);
   QPoint t = GetTopLeftPoint();
   b.moveBy( t.x(), t.y() );
   r->drawFillBox(b.topLeft(), b.bottomRight(), r->getBGColor(), false,
 		 QColor(0,0,0), 1);
+
   // right-align behavior is different for stand-alone text
-  QTextStream dts(&displayText, IO_ReadOnly);
+  QTextStream dts(&displayTextMask, IO_ReadOnly);
   QString displayLine;
   int in1 = 0;
-  displayLine = dts.readLine();
-  in1 = r->GetStringWidth(displayLine, font);
+  //displayLine = dts.readLine();
+  //in1 = r->GetStringWidth(displayLine, font);
   //cout << b.width() << ":" << in1 << endl;
+  in1 = lineWidths[0].toInt();
   in1 = b.width() - in1;
-  bool switchpos = false;
+  //bool switchpos = false;
   QPoint curpos(t.x(), t.y() + r->GetTextHeight(font));
   if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
     curpos.setX(curpos.x() + in1 + 2);
@@ -223,14 +368,21 @@ void Text::Render(void)
   bool smaller_font = false;
   QPoint startpos = curpos;
 
-  int i;
+  // actual rendering section
+
+  int lw1 = 1;
   r->resetTextOrigin();
-  for (i = 0; i < displayText.length(); i++) {
-    if ((int)QChar(displayText[i]) == 10) {
+  for ( QStringList::Iterator it = renderList.begin(); it != renderList.end(); 
+	++it ) {
+    currentElement = *it;
+    if ( currentElement == "</p>" ) {
+      if (justify == JUSTIFY_CENTER) continue;
       r->resetTextOrigin();
-      displayLine = dts.readLine();
-      in1 = r->GetStringWidth(displayLine, font);
+      //displayLine = dts.readLine();
+      //in1 = r->GetStringWidth(displayLine, font);
       //cout << b.width() << ":" << in1 << endl;
+      in1 = lineWidths[lw1].toInt();
+      lw1++;
       in1 = b.width() - in1;
       curpos.setX(t.x());
       if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
@@ -240,157 +392,78 @@ void Text::Render(void)
       curpos.setY(curpos.y() + r->GetTextFullHeight(font));
       continue;
     }
-    if (displayTextMask[i] == ' ')
-      r->drawText(displayText[i], curpos, drawcolor, font);
-    if (displayTextMask[i] == 'B')
-      r->drawText(displayText[i], curpos, drawcolor, bfont);
-    if (displayTextMask[i] == 'I')
-      r->drawText(displayText[i], curpos, drawcolor, ifont);
-    if (displayTextMask[i] == 'U')
-      r->drawText(displayText[i], curpos, drawcolor, ufont);
-    if (displayTextMask[i] == '+') {
-      r->drawText(displayText[i], QPoint(curpos.x(), curpos.y()-supersub), 
-		  drawcolor, supersubfont);
-      smaller_font = true;
-    }
-    if (displayTextMask[i] == '-') {
-      r->drawText(displayText[i], QPoint(curpos.x(), curpos.y()+subx), 
-		  drawcolor, supersubfont);
-      smaller_font = true;
-    }
-    if (smaller_font == true)
-      curpos.setX(curpos.x() + r->GetCharWidth(displayText[i], supersubfont));
-    else
-      curpos.setX(curpos.x() + r->GetCharWidth(displayText[i], font));
-    smaller_font = false;
-  }
-}
 
-void Text::EditRender(void)
-{
-  UpdateDisplayText();
-  int supersub = r->GetTextHeight(font) / 2;
-  QFont supersubfont = subscriptFont(font);
-  int subx = r->GetTextHeight(supersubfont) / 2;
-  QRect b = r->GetTextDimensions(start->element, font);
-  // determine top left point, and start drawing there.
-  QPoint t = GetTopLeftPoint();
-  b.moveBy( t.x(), t.y() );
-  b.setLeft(b.left() - 2); b.setRight(b.right() + 3);
-  b.setTop(b.top() - 1); b.setBottom(b.bottom() + 1);
-  r->drawFillBox(b.topLeft(), b.bottomRight(), r->getBGColor(), true,
-		   QColor(0,0,0), 1);
-  // right-align behavior is different for stand-alone text
-  QTextStream dts(&displayText, IO_ReadOnly);
-  QString displayLine;
-  int in1 = 0;
-  displayLine = dts.readLine();
-  in1 = r->GetStringWidth(displayLine, font);
-  //cout << displayLine << "-" << b.width() << ":" << in1 << endl;
-  in1 = b.width() - in1 - 5;
-  QPoint curpos(t.x(), t.y() + r->GetTextHeight(font));
-  if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    curpos.setX(curpos.x() + in1 + 2);
-  if ( (tjustify == TEXT_CALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    curpos.setX( curpos.x() + (in1 / 2) + 2);
-
-  int i, cnt = 0;
-  bool smaller_font = false;
-  for (i = 0; i < start->element.length(); i++) {
-    if ((int)QChar(start->element[i]) == 10) {
-      cnt++;
-      displayLine = dts.readLine();
-      in1 = r->GetStringWidth(displayLine, font);
-      //cout << displayLine << "-" << b.width() << ":" << in1 << endl;
-      in1 = b.width() - in1 - 5;
-      curpos.setX(t.x());
-      if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
-	curpos.setX(curpos.x() + in1 + 2);
-      if ( (tjustify == TEXT_CALIGN) && (justify == JUSTIFY_TOPLEFT) )
-	curpos.setX( curpos.x() + (in1 / 2) + 2);
-      curpos.setY(curpos.y() + r->GetTextFullHeight(font));
+    if (currentElement.left(6) == "<span ") {
+      // set font
+      //std::cout << "span: " << currentElement << std::endl;
+      boldFlag = false, italicFlag = false, underlineFlag = false,
+	superFlag = false, subFlag = false;      
+      if (currentElement.contains("font-family") > 0) {
+	lp2 = currentElement.find("font-family");
+	lp1 = currentElement.find(":", lp2);
+	lp2 = currentElement.find(";", lp1);
+	currentFont.setFamily( currentElement.mid(lp1 + 1, lp2 - lp1 - 1) );
+      }
+      if (currentElement.contains("font-size") > 0) {
+	lp2 = currentElement.find("font-size");
+	lp1 = currentElement.find(":", lp2);
+	lp2 = currentElement.find("pt", lp1);
+	currentFont.setPointSize( currentElement.mid(lp1 + 1, lp2 - lp1 - 1).toInt() );
+      }
+      currentFont.setBold(false);
+      currentFont.setItalic(false);
+      currentFont.setUnderline(false);
+      if (currentElement.contains("font-weight") > 0) {
+	boldFlag = true;
+	currentFont.setBold(true);
+      }
+      if (currentElement.contains("italic") > 0) {
+	italicFlag = true;
+	currentFont.setItalic(true);
+      }
+      if (currentElement.contains("underline") > 0) {
+	underlineFlag = true;
+	currentFont.setUnderline(true);
+      }
+      if (currentElement.contains("super") > 0) {
+	superFlag = true;
+      }
+      if (currentElement.contains("sub") > 0) {
+	subFlag = true;
+      }
+      currentColor = QColor(0,0,0);
+      if (currentElement.contains("color") > 0) {
+	lp2 = currentElement.find("color");
+	lp1 = currentElement.find(":", lp2);
+	currentColor.setNamedColor( currentElement.mid(lp1 + 1, 7) );
+      }
       continue;
     }
-    if ( (cnt < selectMin) || (cnt > selectMax) ) {
-      if (start->elementmask[i] == ' ')
-	r->drawText(start->element[i], curpos, color, font);
-      if (start->elementmask[i] == 'B')
-	r->drawText(start->element[i], curpos, color, bfont);
-      if (start->elementmask[i] == 'I')
-	r->drawText(start->element[i], curpos, color, ifont);
-      if (start->elementmask[i] == 'U')
-	r->drawText(start->element[i], curpos, color, ufont);
-      if (start->elementmask[i] == '+') {
-	r->drawText(start->element[i], QPoint(curpos.x(), curpos.y()-supersub), 
-		    color, supersubfont);
-	smaller_font = true;
-      }
-      if (start->elementmask[i] == '-') {
-	r->drawText(start->element[i], QPoint(curpos.x(), curpos.y()+subx), 
-		    color, supersubfont);
-	smaller_font = true;
-      }
-    } else {
-      if (start->elementmask[i] == ' ')
-	r->drawTextReverse(start->element[i], curpos, color, font);
-      if (start->elementmask[i] == 'B')
-	r->drawTextReverse(start->element[i], curpos, color, bfont);
-      if (start->elementmask[i] == 'I')
-	r->drawTextReverse(start->element[i], curpos, color, ifont);
-      if (start->elementmask[i] == 'U')
-	r->drawTextReverse(start->element[i], curpos, color, ufont);
-      if (start->elementmask[i] == '+') {
-	r->drawTextReverse(start->element[i], QPoint(curpos.x(), curpos.y()-supersub), 
-			   color, supersubfont);
-	smaller_font = true;
-      }
-      if (start->elementmask[i] == '-') {
-	r->drawTextReverse(start->element[i], QPoint(curpos.x(), curpos.y()+subx), 
-			   color, supersubfont);
-	smaller_font = true;
-      }
-    }
-    cnt++;
-    if (smaller_font == true)
-      curpos.setX(curpos.x() + r->GetCharWidth(start->element[i], supersubfont));
-    else
-      curpos.setX(curpos.x() + r->GetCharWidth(start->element[i], font));
-    smaller_font = false;
-    //curpos.setX(curpos.x() + r->GetCharWidth(start->element[i], font));
-  }
-  // determine cursor position
-  int lx = t.x(), ly = 1;
-  in1 = 0;
-  QTextStream dtsc(&displayText, IO_ReadOnly);
-  displayLine = dtsc.readLine();
-  in1 = r->GetStringWidth(displayLine, font);
-  if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    in1 = b.width() - in1 - 3;
-  if ( (tjustify == TEXT_CALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    in1 = (b.width() - in1 - 3) / 2;
 
-  for (i = 0; i < cursor; i++) {
-    if ((int)QChar(start->element[i]) == 10) {
-      lx = t.x();
-      ly++;
-      displayLine = dtsc.readLine();
-      in1 = r->GetStringWidth(displayLine, font);
-      if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
-	in1 = b.width() - in1 - 3;
-      if ( (tjustify == TEXT_CALIGN) && (justify == JUSTIFY_TOPLEFT) )
-	in1 = (b.width() - in1 - 3) / 2;
-    } else {
-      lx = lx + r->GetCharWidth(start->element[i], font);
+    supersub = r->GetTextHeight(currentFont) / 2;
+    supersubfont = subscriptFont(currentFont);
+    subx = r->GetTextHeight(supersubfont) / 2;
+
+    if (highlighted) currentColor = QColor(255,0,0);
+
+    if (superFlag == true) {
+      r->drawString(currentElement, QPoint(curpos.x(), curpos.y()-supersub), 
+		    currentColor, supersubfont);
+      smaller_font = true;
     }
+    if (subFlag == true) {
+      r->drawString(currentElement, QPoint(curpos.x(), curpos.y()+subx), 
+		    currentColor, supersubfont);
+      smaller_font = true;
+    }
+    if (smaller_font == true) {
+      curpos.setX(curpos.x() + r->GetStringWidth(currentElement,supersubfont));
+    } else {
+      r->drawString(currentElement, curpos, currentColor, currentFont);
+      curpos.setX(curpos.x() + r->GetStringWidth(currentElement,currentFont));
+    }
+    smaller_font = false;
   }
-  if ( (tjustify == TEXT_RALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    lx += in1;
-  if ( (tjustify == TEXT_CALIGN) && (justify == JUSTIFY_TOPLEFT) )
-    lx += in1;
-  ly = t.y() + ((ly - 1) * r->GetTextFullHeight(font)) +
-    r->GetTextHeight(font);
-  r->drawLine(QPoint(lx, ly), QPoint(lx, ly - r->GetTextHeight(font)), 1,
-	      QColor(0,0,0));
 }
 
 int Text::Type(void)
@@ -587,7 +660,15 @@ void Text::InsertCharacter(QKeyEvent *k1) {
   }
 
   // if key > 96 and not already handled, ignore (letters/numbers/space < 96)
-  if (k1->key() > 96) return;
+  // note that this breaks international layouts and is probably not needed
+  //if (k1->key() > 96) return;
+  //
+  // this is a more useful test in a Unicode world
+  //std::cout << "Key text:" << k1->text() << ":" << std::endl;
+  QChar kc1 = k1->text().at(0);
+  if (kc1.isPrint() == false) return;
+  //std::cout << kc1 << "(key accepted)" << std::endl;
+
   // regular letter/number pressed
   start->element.insert(cursor, k1->text());
   //start->element = text;
@@ -595,6 +676,13 @@ void Text::InsertCharacter(QKeyEvent *k1) {
   cursor++;
 }
 
+void Text::InsertString(QString pt) {
+  start->element.insert(cursor, pt);
+  pt.fill(' ');
+  start->elementmask.insert(cursor, pt);
+  cursor += pt.length();
+}
+
 // Superscript selected text
 void Text::DoSuperscript() {
   if (selectMin < 0) return;
@@ -824,6 +912,11 @@ Drawable * Text::DeepCopy() {
 }
 
 void Text::UpdateDisplayText() {
+  return;
+  // this function doesn't do anything now that we're using RichText
+
+  // old code follows
+
   // use user string if justify chosen
   if (tjustify != TEXT_AUTO) {
     displayText = start->element;
