@@ -17,13 +17,14 @@ class XRuler : public QWidget {
   public slots:
     void setHV(int);
   void setRange(int, int);
+  void zoomEvent();
 
  protected:
   void paintEvent(QPaintEvent *);
 
  private:
   int ix, iy, iu, ihv, incr;
-    double dx, dy;
+    double dx, dy, zf;
 };
 
 #endif
